/***************************************************************************
                          attribute.cpp  -  description
                             -------------------
    begin                : Tue Apr 23 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "attribute.h"
#include "data.h"
#include "grapheditview.h"
#include "basicelement.h"

#include <qpainter.h>
#include <qpalette.h>
#include <qheader.h>
#include <qpushbutton.h>
#include <qhbox.h>
#include <qlayout.h>

//#include <qtooltip.h>

Attribute::Attribute(const int ID, GraphEditView * parent, QString name, QString value)
  : QListViewItem(parent, name),
    basicElementID(ID),
    even(false),
    editable(false),
    attribute(0),
    resetButton(0),
    editor(0),
    editorBox(0),
    valString(value),
    graphEditView(parent)
{
}
Attribute::Attribute(const int ID, Attribute * parent, QString name , QString value)
  : QListViewItem(parent, name, value),
    basicElementID(ID),
    even(false),
    editable(false),
    attribute(0),
    resetButton(0),
    editor(0),
    editorBox(0),
    valString(value)
{
  graphEditView = dynamic_cast<GraphEditView *>(listView());
}
Attribute::~Attribute(){
//  cerr << " delete attribute\n";
  if(editorBox){ delete editorBox; editorBox=0;}
}
void Attribute::updateBasicElement(){
  bool disposible = graphEditView->getDisposible();
  graphEditView->setDisposible(false);
  graphEditView->refresh(basicElementID);
  graphEditView->setDisposible(disposible);
}
void Attribute::valueChanged(){
  Attribute * iter = dynamic_cast<Attribute *>(firstChild());
  while(iter){
    iter->valueChanged();
    iter = dynamic_cast<Attribute *>(iter->nextSibling());
  }
}
QColor Attribute::backgroundColor()
{
  if ( graphEditView->firstChild() == this ){
    even = false;
  } else {
    even = ! dynamic_cast<Attribute*>(itemAbove()) -> even;
  }
  if( isSelected() ){
//    return QColor(200,0,180, QColor::Hsv);
    return even ? QColor(200,150,200, QColor::Hsv) : QColor(25,60,200, QColor::Hsv);
  } else {
    return even ? QColor(200,50,200, QColor::Hsv) : QColor(25,20,200, QColor::Hsv);
  }
}
void Attribute::paintCell( QPainter *p, const QColorGroup &cg, int column, int width, int align )
{
//  QRect rect = graphEditView->itemRect( this );
//  QToolTip::remove(graphEditView, rect);
//  QToolTip::add(graphEditView, rect, valString);
//  QListViewItem::paintCell( p, cg, column, width, align);
//  return;
//  cerr << " repainting the cell with " << val\n";
  QColor bgColor = backgroundColor();
  QColorGroup g( cg );
  g.setColor( QColorGroup::Base, bgColor );
  g.setColor( QColorGroup::Background, bgColor );
  g.setColor( QColorGroup::Foreground, Qt::black );
  g.setColor( QColorGroup::Text, Qt::black );

  p->setBrush( cg.base() );
  p->save();

  int indent=0;

  if( ! column ){
    p->fillRect(0, 0, width, height(), bgColor);
    indent = depth() * 14 + 7 - EDITOR_ATTRIBUTE_MARGIN;
    p->translate(indent, 0);
  }

  bool sel = isSelected();
  int halfHeight = height()/2;
  setSelected(false);
  if(editable){
    p->save();
    QFont f = p->font();
    f.setBold( TRUE );
    p->setFont( f );
  }
  QListViewItem::paintCell( p, g, column, width - indent, align  );
  if(editable){
    p->restore();
  }
  setSelected(sel);

  p->restore();
  p->drawLine( 0, height() - 1, width, height() - 1 );
  p->drawLine( width - 1, 0, width - 1, height() );

  if( ! column ){
    if( depth() > 1 ){
      indent = -6;
      for ( int i = 1; i < depth() ; i++ ) {
        p->drawLine( indent, 0 , indent, height() );
        indent += 14;
      }
      if( nextSibling() && nextSibling()->isVisible() ){
        p->drawLine( indent, 0 , indent, height() );
      } else {
        p->drawLine( indent, 0 , indent, halfHeight );
        p->drawLine( indent, halfHeight , indent+6, halfHeight );
      }
      if( firstChild() ){
        p->drawLine( indent+10, halfHeight , indent, halfHeight );
        if ( isOpen() ){
          p->drawLine( indent + 14, height() , indent + 14, halfHeight );
        }
        p->drawRect( indent +  10 , halfHeight - 4, 9, 9 );
      }
    } else if( depth() == 1 && firstChild() ) {
        if( isOpen() )
          p->drawLine( 8, height() , 8, halfHeight );
        p->drawRect( 4 , halfHeight - 4, 9, 9 );
    }
  }
}
void Attribute::createEditorBox(){
  if( ! editorBox ){
    editorBox = new QHBox(graphEditView->viewport());
    editorBox->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
    editorBox->setLineWidth( 1 );
    editorBox->layout()->setAlignment( Qt::AlignRight );
    editorBox->hide();
    graphEditView->addChild( editorBox );

    createEditor();

    resetButton = new QPushButton( "X", editorBox, "reset button" );
    resetButton->setFixedWidth( 20 );
    QObject::connect( resetButton, SIGNAL( clicked() ),  this, SLOT( resetProperty() ) );
  }
}
void Attribute::placeEditor()
{
  if( ! editorBox ) createEditorBox();
/*
  if(valString == defString){
    QColorGroup cg =resetButton->colorGroup();
    cg.setColor(QColorGroup::Text, Qt::red);
    resetButton->setPalette(QPalette(cg,cg,cg));
  }
*/
  QRect r = graphEditView->itemRect( this );
  if ( !r.size().isValid() ) {
    graphEditView->ensureItemVisible( this );
#if defined(Q_WS_WIN)
    graphEditView->repaintContents( false );
#endif
    r = graphEditView->itemRect( this );
  }
  r.setX( graphEditView->header()->sectionPos( 1 ) - graphEditView->contentsX());
  r.setWidth( graphEditView->header()->sectionSize( 1 ) - 1 );
  r.setWidth( r.width() );//- 2 - resetButton->width() );
  r = QRect( graphEditView->viewportToContents( r.topLeft() ), r.size() );
  editorBox->resize( r.size() );
  graphEditView->moveChild( editorBox, r.x(), r.y() );
}
void Attribute::showEditor()
{
  if(editable){
    placeEditor();
    editorBox->show();
    if(editor) editor->setFocus();
  }
}
void Attribute::hideEditor()
{
  if(editorBox) { editorBox->hide(); }
  if(editor)    { this->listView()->setFocus(); }
}
//--------------------------------------Attribute_bool
#include <qpushbutton.h>
#include <qsizepolicy.h>
#include <qfont.h>

Attribute_bool::Attribute_bool(const int ID, Attribute * parent,
                        QString name, bool & val, const bool & def,
                        const BasicElement::EditMode edit)
  : Attribute(ID, parent, name, boolToQString(val)),
    val(val),
    def(def),
    button(0)
{
  editable  = (edit == BasicElement::EDIT);
  valString = boolToQString(val);
  defString = boolToQString(def);
};
void Attribute_bool::createEditor(){
  button = new QPushButton( valString, editorBox, "boolean button" );
//  editor = button;
  button->setAutoDefault(true);
  button->setToggleButton(true);
  button->setOn(val);
//  button->setAccel(Key_Enter);
  QFont font = button->font();
  font.setBold(true);
  button->setFont(font);
  connect( button, SIGNAL( toggled(bool) ), this, SLOT( returnPressed(bool) ) );
  button->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
  button->show();
}
void Attribute_bool::resetProperty(){
  if(button) button->setOn(def);
  returnPressed(def);
}
void Attribute_bool::returnPressed(bool on){
//  cerr << " bool return pressed" << ( on ? " true\n " : " false\n") ;
  val = on;
  valString = boolToQString(val);
  this->setText(1,valString);
  if(button){
    button->setText(valString);
    button->setOn(on);
  }
  updateBasicElement();
}
void Attribute_bool::showEditor(){
  if( ! editable) return;
//  cerr << "  show editor " << ( val ? " true\n " : " false\n") ;
  Attribute::showEditor();
  Attribute_bool::returnPressed( val ? false : true );
}
//void Attribute_bool::hideEditor(){
//  if(editor) { this->listView()->setFocus();}
//}
void Attribute_bool::valueChanged(){
  valString = boolToQString(val);
  this->setText(1,valString);
  if(button) button->setText(valString);
}

//--------------------------------------Attribute_enum

#include <qcombobox.h>
#include <qsizepolicy.h>

#include "domenumeration.h"

Attribute_enum::Attribute_enum(const int ID, Attribute * parent, DomEnumeration * enumeration,
                                  QString name, int & val, const int & def,
                                  const BasicElement::EditMode edit)
  : Attribute(ID, parent, name, enumeration->stringAtValue(val) ),
    enumeration(enumeration),
    val(val),
    controlVal(val),
    def(def)
{
  editable  = (edit == BasicElement::EDIT || edit == BasicElement::SHOW || edit == BasicElement::OPEN);
  valString = enumeration->stringAtValue(val);
  defString = enumeration->stringAtValue(def);
};
void Attribute_enum::createEditor(){
  editor = new QComboBox( false, editorBox );
  for(int index = 0; index < enumeration->size(); index++){
    ((QComboBox*)editor)->insertItem(enumeration->stringAtIndex(index));
  }
 ((QComboBox*)editor)->setCurrentItem(enumeration->indexOfValue(val));
//  cerr << " val: " << val << "current item: "  << enumeration->indexOfValue(val) << endl;
  connect( ((QComboBox*)editor), SIGNAL( activated( int ) ), this, SLOT( returnPressed(int) ) );
  editor->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
  editor->show();
}
void Attribute_enum::resetProperty(){
  returnPressed( def );
}
void Attribute_enum::returnPressed(int index){
  val = enumeration->valueAtIndex(index);
  valString = enumeration->stringAtValue(val);
  this->setText(1,valString);
  setSelected(false);
  hideEditor();
  updateBasicElement();
}
void Attribute_enum::hideEditor(){
  if(editorBox){
    if(editor)
      ((QComboBox*)editor)->setCurrentItem(enumeration->indexOfValue(val));
  }
  Attribute::hideEditor();
}
void Attribute_enum::valueChanged(){
  valString = enumeration->stringAtValue(val);
  this->setText(1,valString);
  if(editorBox){
    if(editor)
      ((QComboBox*)editor)->setCurrentItem(enumeration->indexOfValue(val));
  }
//  QListViewItem * child = firstChild();
//  while ( child ) {
//    child->setVisible(false);
//    child = child->nextSibling();
//  }
}

//--------------------------------------Attribute_select

#include "basicelement.h"
#include "domenumeration.h"
#include <qcombobox.h>
#include <qsizepolicy.h>

Attribute_select::Attribute_select(const int ID, Attribute * parent, DomEnumeration * enumeration,
                                      QString name, int & val, const int & def,
                                      const BasicElement::EditMode edit)
  : Attribute(ID, parent, name, (val == -1 ? QString(" --- ") : enumeration->stringAtValue(val) ) ),
    enumeration(enumeration),
    val(val),
    def(def)
{
  editable  = (edit == BasicElement::EDIT || edit == BasicElement::SHOW);
  valString = (val == -1 ? QString(" --- ") : enumeration->stringAtValue(val) );
  defString = (def == -1 ? QString(" --- ") : enumeration->stringAtValue(def) );
};
void Attribute_select::createEditor(){
  editor = new QComboBox( false, editorBox );
  ((QComboBox*)editor)->insertItem(" --- ");
  for(int index = 0; index < enumeration->size(); index++){
    ((QComboBox*)editor)->insertItem(enumeration->stringAtIndex(index));
  }
  ((QComboBox*)editor)->setCurrentItem(1 + enumeration->indexOfValue(val));
  connect( ((QComboBox*)editor), SIGNAL( activated( int ) ), this, SLOT( returnPressed(int) ) );
  editor->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
  editor->show();
}
void Attribute_select::resetProperty(){
  returnPressed( def );
}
void Attribute_select::returnPressed(int index){
  val = enumeration->valueAtIndex(index - 1);
  valString = (val == -1 ? QString(" --- ") : enumeration->stringAtValue(val) );
  this->setText(1,valString);
  setSelected(false);
  hideEditor();
  QListViewItem * child = firstChild();
  while ( child ) {
    child->setVisible(false);
    child = child->nextSibling();
  }
  updateBasicElement();
}
void Attribute_select::hideEditor(){
  if(editorBox){
    if(editor)
      ((QComboBox*)editor)->setCurrentItem(1 + enumeration->indexOfValue(val));
  }
  Attribute::hideEditor();
}
void Attribute_select::valueChanged(){
  valString = (val == -1 ? QString(" --- ") : enumeration->stringAtValue(val) );
  this->setText(1,valString);
  if(editorBox){
    if(editor)
      ((QComboBox*)editor)->setCurrentItem(1 + enumeration->indexOfValue(val));
  }
  QListViewItem * child = firstChild();
  while ( child ) {
    child->setVisible(false);
    child = child->nextSibling();
  }
  if(enumeration->elementAtValue(val)){
    enumeration->elementAtValue(val)->xml_edit(this, false);
  }
}
