/***************************************************************************
                          attribute.h  -  description
                             -------------------
    begin                : Tue Apr 23 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ATTRIBUTE_H
#define ATTRIBUTE_H


#define EDITOR_ATTRIBUTE_MARGIN 5

#include "basicelement.h"
#include "grapheditview.h"

#include <qpalette.h>
#include <qobject.h>

class Collection;

class QPainter;
class QColor;
class QPushButton;
class QDomNode;
class QHBox;

class Attribute : public QObject,  public QListViewItem  {
  Q_OBJECT

public:
  Attribute(const int ID, GraphEditView * parent, QString name, QString value = QString::null);
  Attribute(const int ID, Attribute * parent, QString name , QString value = QString::null);
  ~Attribute();

public:
  void paintCell( QPainter *p, const QColorGroup &cg, int column, int width, int align );
  void createResetButton();
  void setAttribute(QDomNode * attr){ attribute = attr; }
  virtual void createEditorBox();
  virtual void createEditor(){;}
  virtual void placeEditor();
  virtual void showEditor();
  virtual void hideEditor();

public slots:
  virtual void resetProperty(){;}
  virtual void valueChanged();

protected:
  QColor backgroundColor();
  void updateBasicElement();

protected:
  const int basicElementID;
  bool even;
  bool editable;
  QDomNode * attribute;
  QPushButton * resetButton;
  QWidget * editor;
  QHBox * editorBox;
  QString valString;
  QString defString;
  GraphEditView * graphEditView;
};

//--------------------------------------Attribute_bool

class Attribute_bool: public Attribute{
  Q_OBJECT

public:
  Attribute_bool(const int ID, Attribute * parent, QString name,
                 bool & val, const bool & def,
                 const BasicElement::EditMode edit = BasicElement::SHOW);

public:
  virtual void createEditor();
  virtual void showEditor();
//  virtual void hideEditor();
  virtual void valueChanged();

public slots:
  virtual void resetProperty();
  virtual void returnPressed(bool on);

private:
  bool &val;
  bool def;
  QPushButton * button;
};

//--------------------------------------Attribute_enum

class QComboBox;
class DomEnumeration;

class Attribute_enum: public Attribute{
  Q_OBJECT

public:
  Attribute_enum(const int ID, Attribute * parent, DomEnumeration *, QString, int &, const int & ,
                  const BasicElement::EditMode edit = BasicElement::SHOW);

public:
  virtual void createEditor();
//  virtual void showEditor();
  virtual void hideEditor();
  virtual void valueChanged();

public slots:
  virtual void resetProperty();
  virtual void returnPressed(int index=0);

private:
  DomEnumeration * enumeration;
  QComboBox * combo;
  int &val;
  int controlVal;
  int def;
};

//--------------------------------------Attribute_select

class QComboBox;
class BasicElement;

class Attribute_select: public Attribute{
  Q_OBJECT

public:
  Attribute_select(const int ID, Attribute * parent, DomEnumeration * enumeration, QString, int &,
                    const int & , const BasicElement::EditMode edit = BasicElement::SHOW);
public:
  virtual void createEditor();
//  virtual void showEditor();
  virtual void hideEditor();
  virtual void valueChanged();

public slots:
  virtual void resetProperty();
  virtual void returnPressed(int index=1);

private:
  DomEnumeration * enumeration;
  QComboBox * combo;
  int &val;
  int def;
};

#endif
