/***************************************************************************
                          attribute_line_edit.cpp  -  description
                             -------------------
    begin                : Tue Apr 23 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "attribute_line_edit.h"
#include "basicelement.h"

#include <qvalidator.h>
#include <qlineedit.h>
#include <qstring.h>
#include <qhbox.h>

//--------------------------------------Attribute_line_edit
Attribute_line_edit::Attribute_line_edit(const int ID, Attribute * parent,
                                         QString name, QString value,
                                         BasicElement::EditMode edit)
  : Attribute(ID, parent, name, value)
{
  editable = (edit == BasicElement::EDIT);
}
void Attribute_line_edit::createEditor(){
  lineEdit = new QLineEdit(valString, editorBox, "line editor");
  editor = lineEdit;
  QObject::connect(lineEdit, SIGNAL(returnPressed()), this, SLOT(returnPressed()));
  createValidator();
}
void Attribute_line_edit::hideEditor()
{
  if(editorBox){
    if(editor) lineEdit->setText(valString);
  }
  Attribute::hideEditor();
}
void Attribute_line_edit::resetProperty(){
  if(editor) lineEdit->setText(defString);
  returnPressed();
}
void Attribute_line_edit::returnPressed(){
  if(editor) valString = lineEdit->text();
  this->setText(1,valString);
  hideEditor();
  setSelected(false);
}
//--------------------------------------Attribute_int
Attribute_int::Attribute_int(const int ID, Attribute * parent,
                             QString name, int &val, const int & def,
                             BasicElement::EditMode edit)
  : Attribute_line_edit(ID, parent, name, QString::number(val), edit),
    val(val),
    def(def)
{
  valString = QString::number(val);
  defString = QString::number(def);
};
void Attribute_int::createValidator(){
  QIntValidator * range = new QIntValidator(editorBox);
  lineEdit->setValidator(range);
}
void Attribute_int::returnPressed(){
  Attribute_line_edit::returnPressed();
  val = valString.toInt();
  updateBasicElement();
}
void Attribute_int::valueChanged(){
  valString = QString::number(val);
  this->setText(1,valString);
}
//--------------------------------------Attribute_double
Attribute_double::Attribute_double(const int ID, Attribute * parent,
                                   QString name, double & val, const double & def,
                                   BasicElement::EditMode edit)
  : Attribute_line_edit(ID, parent, name, QString::number(val), edit),
    val(val),
    def(def)
{
  valString = QString::number(val);
  defString = QString::number(def);
};
void Attribute_double::createValidator(){
  QDoubleValidator * range = new QDoubleValidator(editorBox);
  lineEdit->setValidator(range);
}
void Attribute_double::returnPressed(){
  Attribute_line_edit::returnPressed();
  val = valString.toDouble();
  updateBasicElement();
}
void Attribute_double::valueChanged(){
  valString = QString::number(val);
  this->setText(1,valString);
}
//--------------------------------------Attribute_qstring
Attribute_qstring::Attribute_qstring(const int ID, Attribute * parent,
                                     QString name, QString & val,
                                     const QString def,
                                     BasicElement::EditMode edit)
  : Attribute_line_edit(ID, parent, name, val, edit),
  val(val),
  def(def)
{
  valString = val;
  defString = def;
};
void Attribute_qstring::returnPressed(){
  Attribute_line_edit::returnPressed();
  val = valString;
  updateBasicElement();
}
void Attribute_qstring::valueChanged(){
  valString = val;
  this->setText(1,valString);
}

