/***************************************************************************
                          axes.cpp  -  description
                             -------------------
    begin                : Wed Mar 6 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "axes.h"
#include "ruler.h"
#include "data.h"
#include "scientificcanvas.h"

#include <qdom.h>
#include <qevent.h>

#include <cmath>

using namespace std;

Axes::CenterEnumeration Axes::centerEnumeration;

Axes::CenterEnumeration::CenterEnumeration()
  : DomEnumeration("axes enumeration")
{
  insert("lower left", LOWERLEFT);
  insert("lower right", LOWERRIGHT);
  insert("center", CENTER);
  insert("zero", ZERO);
//insert("offset", OFFSET);
  insert("manual", MANUAL);
}

Axes::Axes(ScientificCanvas * canvas, const QString name, QDomElement * element)
  : BasicElement(canvas ? canvas->getID(): 0, name),
    QCanvasLine(canvas),
    canvas(canvas),
    horizontal(canvas ? canvas->getID(): 0, canvas, "horizontal ruler"),
    vertical(canvas ? canvas->getID(): 0, canvas, "vertical ruler"),
    centralPoint(canvas ? canvas->getID(): 0, canvas)
{
  hideName();
  connect( & horizontal,  SIGNAL(  invTangensChanged(    double)),
           & vertical,    SLOT(    setTangens(    double)));
  connect( & vertical,    SIGNAL(  invTangensChanged(    double)),
           & horizontal,  SLOT(    setTangens(    double)));
  xml_read(element);
}
bool Axes::xml_read(QDomElement * element){
  bool result = BasicElement::xml_read(element);
  if(element){
    QDomNodeList list = element->elementsByTagName("Ruler");
    makeXML_Element( & horizontal, & list);
    makeXML_Element( & vertical, & list);
  } 
  controlLeftX = leftX;
  controlRightX = rightX;
  controlUpperY = upperY;
  controlLowerY = lowerY;
  setID(getID());
//  cerr << "lowerY " << lowerY << " upperY " << upperY << " leftX " << leftX << " rightX " << rightX << "\n";
  return result;
}
void Axes::xml_list(){
  BasicElement::xml_list();
  xml_bool("valid", valid, true, HIDE);
  xml_bool("visible", visible, true, EDIT);
  xml_bool("numbers", numbers, true, EDIT);
  xml_bool("validSteps", validSteps, true, HIDE);
  xml_qstring("color", colorName, "navy", EDIT);
  xml_int("angle", angle, 0, HIDE);
  xml_int("originX", originX, 0, HIDE);
  xml_int("originY", originY, 0, HIDE);
  xml_double("offsetX", offsetX, 0, (center == OFFSET ? EDIT : HIDE) );
  xml_double("offsetY", offsetY, 0, (center == OFFSET ? EDIT : HIDE) );
  // the ORIGIN is where the axes and all functions have there Zero on the canvas.
  // the OFFSET is where the axes may have there cross, which may be at another point than 0:0
  xml_double("lowerY", lowerY, -10, EDIT);
  xml_double("upperY", upperY, 10, EDIT);
  xml_double("leftX", leftX, 0, EDIT);
  xml_double("rightX", rightX, 10, EDIT);
  xml_enum( & centerEnumeration, "center", center, LOWERLEFT);
  xml_object( & vertical);
  xml_object( & horizontal);
}
void Axes::setValid(const bool yes){
  valid = yes;
  if( ! valid ) refresh();
}
void Axes::refresh(){
//  cerr << " --------------refresh ----- " << "leftX " << leftX << " rightX " << rightX << " lowerY "  << lowerY << " upperY " << upperY << endl;
  Scale * scale = horizontal.getScale();
  scale->refresh();
  if( (leftX  < scale->lowerBound()) )  leftX = scale->lowerBound();
  if( (rightX > scale->upperBound()) )  rightX = scale->upperBound();
  scale = vertical.getScale();
  scale->refresh();
//  cerr << " ------------refresh 2---- " << "leftX " << leftX << " rightX " << rightX << " lowerY "  << lowerY << " upperY " << upperY << endl;
  if( (lowerY < scale->lowerBound()) )  lowerY = scale->lowerBound();
  if( (upperY > scale->upperBound()) )  upperY = scale->upperBound();

  if( controlLeftX != leftX || controlRightX != rightX || controlUpperY != upperY || controlLowerY != lowerY){
    valid = false;
    controlLeftX = leftX;
    controlRightX = rightX;
    controlUpperY = upperY;
    controlLowerY = lowerY;
  }
  horizontal.setNumbers(numbers);      // no dependencies
  vertical.setNumbers(numbers);        // no dependencies
  setPen(QPen(QColor(colorName)));      // no dependencies
  setVisible(visible);                   // no dependencies
  setMargin(margin);                     // to avoid dev. by zero this always leaves space in the center.
  recalculate();                 // depends on margin
  setOrigin();                           // depends on margin and calcVar()
  setAngle(angle);                       // calls rule.setTangens() and subsequently rule.refresh()
  if( ! valid ){
    valid = true;
    canvas->setValid(false);
  }
  canvas->update(); 
//  cerr << " ------after---refresh ----- " << "leftX " << leftX << " rightX " << rightX << " lowerY "  << lowerY << " upperY " << upperY << endl;
}
void Axes::recalculate(){  
//  cerr << "- before recalculate \n";
//  cerr << lowerY " << lowerY << " upperY " << upperY << " leftX " << leftX << " rightX " << rightX << "\n";

  leftVisibleX = margin;
  rightVisibleX = canvas->width() - margin;
  upperVisibleY = margin;
  lowerVisibleY = canvas->height() - margin;

  leftPosX = horizontal.getScale()->fun(leftX);
  rightPosX = horizontal.getScale()->fun(rightX);
  lowerPosY = vertical.getScale()->fun(lowerY);
  upperPosY = vertical.getScale()->fun(upperY);
  validSteps = true;
  stepX = (rightPosX - leftPosX) / (double(rightVisibleX - leftVisibleX)); // no zeros
  if( ! stepX ) validSteps = false;
  stepY = (upperPosY - lowerPosY) / (double(lowerVisibleY - upperVisibleY)); // no zeros
  if( ! stepY ) validSteps = false;
//  cerr << " ----------------- " << "leftX " << leftX << " rightX " << rightX << " lowerY "  << lowerY << " upperY " << upperY << endl;
//  cerr << "leftPosX " << leftPosX << " rightPosX " << rightPosX << " lowerPosY "  << lowerPosY << " upperPosY " << upperPosY << endl;
}
bool Axes::valToPos(const double & valX, const double & valY, int & posX, int & posY){
  if( ! validSteps)
    return false;
  bool result = true;
  posX = (int)floor(((horizontal.getScale()->fun(valX) - leftPosX) / stepX) + leftVisibleX + .5);
  posY = (int)floor(lowerVisibleY - ((vertical.getScale()->fun(valY) - lowerPosY) / stepY));
  if(posX < leftVisibleX)  { posX = leftVisibleX;  result = false; }
  if(posX > rightVisibleX) { posX = rightVisibleX; result = false; }
  if(posY < upperVisibleY) { posY = upperVisibleY; result = false; }
  if(posY > lowerVisibleY) { posY = lowerVisibleY; result = false; }
//  cerr << " valX " << valX  << " fun(valX) " << horizontal.getScale()->fun(valX) << " stepX " << stepX << " leftPosX " << leftPosX << " leftVisX " << leftVisibleX << endl;
//  cerr << " valY " << valY << " fun(valY) " << vertical.getScale()->fun(valY) << " stepY " << stepY << " lowerPosY " << lowerPosY << " lowerVisY " << lowerVisibleY << endl;
  return result;
}
bool Axes::posToVal(const int & posX, const int & posY, double & valX, double & valY){
  if(posX < leftVisibleX || posX > rightVisibleX || posY < upperVisibleY || posY > lowerVisibleY){
    return false;
  } else {
    valX = horizontal.getScale()->inv( ((posX - leftVisibleX) * stepX)  + leftPosX );
    valY = vertical.getScale()->inv( ((lowerVisibleY - posY) * stepY) + lowerPosY );
//  cerr << " ----------- valX " << valX << " valY " << valY << " x " << posX << " y " << posY << endl;
//  cerr << " ----------- inv(valX) " << horizontal.getScale()->inv(valX) << " stepX " << stepX << " leftPosX " << leftPosX << " leftVisX " << leftVisibleX << endl;
    return true;
  }
}
void Axes::setVisible(const bool yes){
  visible = yes;
  vertical.setVisible(yes);
  horizontal.setVisible(yes);
  return;
}
void Axes::setOrigin(){
  switch(center){
    case MANUAL:
      posToVal(originX, originY, offsetX, offsetY);
      break;
    case LOWERLEFT:
      originX = margin;
      originY = margin;
      posToVal(originX, originY, offsetX, offsetY);
      break;
    case LOWERRIGHT:
      originX = canvas->width() - margin;
      originY = margin;
      posToVal(originX, originY, offsetX, offsetY);
      break;
    case CENTER:
      originX = canvas->width()/2;
      originY = canvas->height()/2;
      posToVal(originX, originY, offsetX, offsetY);
      break;
    case ZERO:
      valToPos(0, 0, originX, originY);
      posToVal(originX, originY, offsetX, offsetY);
      break;
    case OFFSET:
      valToPos(offsetX, offsetY, originX, originY);
      break;
  }
  canvas->emitValueChanged();
//  cerr << " originX " << originX << " originY " << originY << endl;
  horizontal.setOrigin( originX , originY);
  vertical.setOrigin( originX, originY);
  centralPoint.setPoints(originX, originY, originX, originY);
}
void Axes::setMargin(int newMargin) {
  if(newMargin >= canvas->height()/2)
    newMargin = canvas->height()/2 - 1;
  if(newMargin >= canvas->width()/2)
    newMargin = canvas->width()/2 - 1;
  margin = newMargin;
  vertical.setMargin(margin);
  horizontal.setMargin(margin);
}
void Axes::advance(int phase){
  if (phase) setAngle(++angle);
}
void Axes::setPen(const QPen & newPen){
  vertical.setPen(newPen);
  horizontal.setPen(newPen);
  centralPoint.setPen(newPen);
}
void Axes::setAngle(const int & newAngle){
  angle = newAngle % 360;
  horizontal.setAngle(newAngle);
  vertical.setAngle(newAngle+90);
}
void Axes::toggleColor(const bool yes){
  if(yes){
    int h, s, v;
    QColor toggle(colorName);
    toggle.hsv(&h, &s, &v);
    toggle.setHsv((h + 180) % 360, s, v);
    setPen( QPen( toggle ) );
  } else {
    setPen( QPen( QColor(colorName).rgb() ) );
  }
  canvas->update();
}
void Axes::expand(const double & up, const double & down, const double & left, const double & right){
  double myUp = up + (up - down) / 50;
  double myDown = down - (up - down) / 50;
  double myRight = right + (right - left) / 50;
  double myLeft = left - (right - left) /50;
  if( myUp    > upperY ) upperY = myUp;
  if( myDown  < lowerY ) lowerY = myDown;
  if( myLeft  < leftX  ) leftX  = myLeft;
  if( myRight > rightX ) rightX = myRight;
  refresh();
}
void Axes::setCanvasSize(const int newWidth, const int newHeight ){
  width = newWidth;
  height = newHeight;
  horizontal.setCanvasSize( width, height );
  vertical.setCanvasSize( width, height );
}
void Axes::setID(const int ID){
  BasicElement::setID(ID);
  horizontal.setID(ID);
  vertical.setID(ID);
  centralPoint.setID(ID);
}
