/***************************************************************************
                          axes.h  -  description
                             -------------------
    begin                : Wed Mar 6 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef AXES_H
#define AXES_H

#include "macros.h"

#include "ruler.h"
#include "basicelement.h"
#include "scale.h"
#include "domenumeration.h"

#include <qcanvas.h>

class ScientficCanvas;

class QPen;
class QDomElement;
class QString;

class Axes : public BasicElement, public QCanvasLine {

public:

  Axes(ScientificCanvas * canvas, const QString name="axes", QDomElement * element = 0);
  
  virtual const QString tagName() const { return "Axes"; }
  void setMargin(int newMargin = MARGIN_DEFAULT); // defined in ruler.h
  int getMargin() const                                { return margin; }
  void setAngle(const int & angle);
  void show(){ setVisible(true);}
  void hide(){ setVisible(false);}
  double invScaleX(const double & counter)             { return horizontal.getScale()->inv(counter); }
  double invScaleY(const double & counter)             { return vertical.getScale()->inv(counter); }

  bool valToPos(const double & valX, const double & valY, int & posX, int & posY);
  bool posToVal(const int & posX, const int & posY, double & valX, double & valY);

  void setOrigin();
  QPoint getOrigin() const                             { return QPoint(originX, originY);}
  double begin() const                                 { return leftPosX; }
  double end() const                                   { return rightPosX; }
  double step() const                                  { return stepX; }
  void setUpperY(double newY)                          { upperY = newY; }
  void setLowerY(double newY)                          { lowerY = newY; }
  void setRightX(double newX)                          { rightX = newX; }
  void setLeftX(double newX)                           { leftX = newX; }
  double getUpperY()                                   { return upperY; }
  double getLowerY()                                   { return lowerY; }
  double getRightX()                                   { return rightX; }
  double getLeftX()                                    { return leftX; }
  void expand(const double & up, const double & down, const double & left, const double & right);
  void setHorizontalScale(Scale::Method method)        { horizontal.getScale()->setMethod(method); recalculate(); }
  void setVerticalScale(Scale::Method method)          { vertical.getScale()->setMethod(method); recalculate(); }

  void setCanvasSize(const int newWidth, const int newHeight );

public: // overridden
  virtual bool xml_read(QDomElement * element);
  virtual void xml_list();
  virtual void refresh();
  virtual void advance(int phase);
  virtual void setPen(const QPen & pen);
  virtual void setID(const int ID);
  virtual void toggleColor(const bool yes= true);
  virtual void setVisible(const bool yes=true);
  virtual void setValid(const bool yes=true);

private:
  void recalculate();
  bool inWidth(const int & point) const  { return (point >= margin && point <= width-margin);}
  bool inHeight(const int & point) const { return (point >= margin && point <= height-margin);}

private:
  enum Center{
    LOWERLEFT,
    LOWERRIGHT,
    ZERO,
    OFFSET,
    CENTER,
    MANUAL
  };
  
  class CenterEnumeration : public DomEnumeration{
    public: CenterEnumeration();
  };
  static CenterEnumeration centerEnumeration;
  
private:
  ScientificCanvas * canvas;
   
  int center, manualCenter;

  Ruler horizontal;
  Ruler vertical;
  RulerLine centralPoint;

  QString colorName;
  bool visible;
  bool numbers;
  bool validSteps;
  double offsetX, offsetY;
  int originX, originY;
  int manualOriginX, manualOriginY;
  double manualOffsetX, manualOffsetY;
  int leftVisibleX, rightVisibleX, upperVisibleY, lowerVisibleY;
  int margin;
  int angle;
  double ( * valX) (const int & posX);
  double ( * valY) (const int & posY);
  int ( * posX ) (const double & valX);
  int ( * posY ) (const double & valY);
  double leftX, rightX, lowerY, upperY;
  double controlLeftX, controlRightX, controlLowerY, controlUpperY;
  double stepX, stepY;
  double leftPosX, rightPosX, lowerPosY, upperPosY;
  int width, height;

};

#endif
