/***************************************************************************
                          collection.cpp  -  description
                             -------------------
    begin                : Fri May 3 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "collection.h"
#include "sample.h"
#include "polynomial.h"

#include <qdom.h>

Collection::Collection(const char * name)
  : name(name),
    elementCount(0),
    elementToDestroy(0)
{
}
Collection::~Collection(){
  if(elementToDestroy){
    delete elementToDestroy;
    elementToDestroy = 0;
  }
}
QString Collection::IDToName(const int ID) {
  if(size() && ID > 0){
    for(Collection::iterator iter = begin(); iter != end(); iter++){
      if( (*iter)->getID() == ID ) return (*iter)->getElementName();
    }
  }
  return QString::null;
}
BasicElement * Collection::IDToElement(const int ID) {
  if(size() && ID > 0){
    for(Collection::iterator iter = begin(); iter != end(); iter++){
       if( (*iter)->getID() == ID ) return (*iter);
    }
  }
  return (BasicElement*)0;
}
int Collection::elementToID(BasicElement * element){
  if(size()){
    for(Collection::iterator iter = begin(); iter != end(); iter++){
      if( (*iter) == element ) return (*iter)->getID();
    }
  }
  return 0;
}
int Collection::IDToIndex(const int ID){
  if( size() && (ID > 0) ){
    int index=0;
    for(Collection::iterator iter = begin(); iter != end(); iter++, index++){
      if( (*iter)->getID() == ID ) return index;
    }
  }
  return -1;
}
int Collection::IndexToID(const int index){
  if( index >= 0 && index < (int)size()){
    int myIndex=0;
    for(Collection::iterator iter = begin(); iter != end(); iter++, myIndex++){
      if( myIndex == index ) return (*iter)->getID();
    }
  }
  return 0;
}
void Collection::remove(const int ID) {
  if(size()){
    for(Collection::iterator iter = begin(); iter != end(); iter++){
      if( (*iter)->getID() == ID ){
        if(elementToDestroy) {
          delete elementToDestroy;
        }
        elementToDestroy = (*iter);
        erase(iter);
        return;
      }
    }
  }
}
int Collection::push_back(BasicElement * element){
//  cerr << " newGraph " << element->getElementName() << endl;
  element->setParentCollection(this);
  if(element->getID() < 1){
    element->setID(++elementCount);
  } else {
    if(element->getID() > elementCount) elementCount = element->getID();
    else for(Collection::iterator iter = begin(); iter != end(); iter++){
      if( (*iter)->getID() == element->getID() ) {
        element->setID(++elementCount);
        break;
      }
    }
  }
  vector<BasicElement * >::push_back(element);
//  cerr << element->getElementName() << " new ID : " << element->getID() << " size: " << size() << endl;
  return element->getID();
}

