/***************************************************************************
                          statisticaldataviewer.cpp  -  description
                             -------------------
    begin                : Wed Jan 15 2003
    copyright            : (C) 2003 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dataviewer.h"
#include "sdvstatusbar.h"
#include "macros.h"
#include "grapheditor.h"
#include "project.h"
#include "workspace.h"
#include "plot.h" 

#include <qapplication.h>
#include <qsplitter.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qpopupmenu.h>
#include <qtoolbutton.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qstring.h>
#include <qpopupmenu.h>

#include "icons/filenew.xpm"
#include "icons/fileopen.xpm"
#include "icons/filesave.xpm"
#include "icons/processicon.xpm"
#include "icons/polynomialicon.xpm"
//#include "icons/neuralicon.xpm"
#include "icons/sampleicon.xpm"
#include "icons/spliticon.xpm"
#include "icons/plotsave.xpm"
//#include "icons/dataload.xpm"

StatisticalDataViewer::StatisticalDataViewer(QWidget * parent,
            const char * name,
            WFlags f)
  : QMainWindow(parent,name,f),
    name(name),
    currentProject(0)
{
  setCaption("Statistical Data Viewer");

  makeMenus();

  statusBar = new SDVStatusBar(this, "status bar");

  centralFrame = new QSplitter(this,"central frame");
  setCentralWidget(centralFrame);

  graphEditor = new GraphEditor(centralFrame, "project editor");

  centralFrame->setResizeMode(graphEditor, QSplitter::KeepSize);

  QApplication::setGlobalMouseTracking(true);

  workspace = new Workspace(centralFrame, statusBar, "central workspace");
  connect(workspace, SIGNAL(windowActivated(QWidget *)), this , SLOT(windowActivated(QWidget *)));
}
StatisticalDataViewer::~StatisticalDataViewer(){
  destroy();
}
void StatisticalDataViewer::destroy(){
  if(projects.size()){
    for(vector<Project * >::iterator iter=projects.begin(); iter != projects.end(); iter++){
      if(*iter){
        delete (*iter);
      }
    }
    projects.clear();
  }
}
void StatisticalDataViewer::makeMenus(){

//  QAccel *accel = new QAccel( this );
//  accel->connectItem( accel->insertItem( Key_Escape ), qApp, SLOT(quit()) );

  QMenuBar * menu = menuBar();

  QPopupMenu * fileMenu = new QPopupMenu;
  fileMenu->insertItem("&New project", this, SLOT(newProject()));
  fileMenu->insertItem("&Open project", this, SLOT(openProject()));
  fileMenu->insertItem("&Save project", this, SLOT(saveProject()));
  fileMenu->insertItem("&Close project", this, SLOT(closeProject()));
  fileMenu->insertSeparator();
  fileMenu->insertItem("Save &plot", this, SLOT(savePlot()), CTRL + Key_S);
  fileMenu->insertSeparator();
  fileMenu->insertItem("E&xit", qApp, SLOT(quit()), CTRL + Key_Q);
  menu->insertItem("&File",fileMenu);

//  optionsMenu = new QPopupMenu;
//  menu->insertItem("&Options",optionsMenu);

  viewMenu = new QPopupMenu;
  viewMenu->insertItem("&Cascade", this, SLOT(cascade()), CTRL + Key_C);
  viewMenu->insertItem("&Tile", this, SLOT(tile()), CTRL + Key_T);
  viewMenu->setItemChecked(animatedID, false);
  dblBufferID = viewMenu->insertItem("&Fine zoom", this, SLOT(toggleDoubleBuffer()));
  viewMenu->setItemChecked(dblBufferID, false);
  animatedID = viewMenu->insertItem("A&nimate", this, SLOT(toggleAnimated()), CTRL + Key_N);
  menu->insertItem("&View",viewMenu);

  projectMenu = new QPopupMenu;
  menu->insertItem("&Projects", projectMenu);

  menu->insertSeparator();

  QPopupMenu* help = new QPopupMenu;
  help->insertItem("&About", this, SLOT(help()), Key_F1);
  menu->insertItem("&Help",help);

//  menu->insertSeparator();
//  QPopupMenu* test = new QPopupMenu;
//  test->insertItem("T&est", this, SLOT(test()), CTRL + Key_T);
//  menu->insertItem("&Test",test);

  QToolBar * mainToolBar = new QToolBar(this);

  new QToolButton(QPixmap(filenew),      "new project",  QString::null, this, SLOT(newProject()),  mainToolBar, "new project");
  new QToolButton(QPixmap(fileopen),     "open project", QString::null, this, SLOT(openProject()), mainToolBar, "open project");
  new QToolButton(QPixmap(filesave),     "save project", QString::null, this, SLOT(saveProject()), mainToolBar, "save project");
  new QToolButton(QPixmap(plotsave),     "save plot",    QString::null, this, SLOT(savePlot()),    mainToolBar, "save plot");

  mainToolBar->addSeparator();

  new QToolButton(QPixmap(processicon),  "new process",  QString::null, this, SLOT(addProcess()),  mainToolBar, "new process");
  new QToolButton(QPixmap(sampleicon),   "new sample",   QString::null, this, SLOT(addSample()),   mainToolBar, "new sample");
  new QToolButton(QPixmap(polynomialicon), "new polynomial", QString::null, this, SLOT(addPolynomial()), mainToolBar, "new polynomial");
//  new QToolButton(QPixmap(neuralicon),   "new neural network", QString::null, this, SLOT(addNeural()), mainToolBar, "add neural");

  mainToolBar->addSeparator();

  new QToolButton(QPixmap(spliticon),    "new split operator",    QString::null, this, SLOT(addSplit()), mainToolBar, "new split operator");

}

//---------------------------------------------------------------------newProject()
void StatisticalDataViewer::newProject(){
  currentProject = new Project(workspace, Project::makeName("project", false, (vector<BasicElement * > *) &projects));
  currentProject->init();
  makeProject();
}
//---------------------------------------------------------------------openProject()
void StatisticalDataViewer::openProject(){
  Project * oldProject = currentProject;
  currentProject = new Project(workspace, "project");
  if (currentProject->load()){
    makeProject();
  } else {
    delete currentProject;
    currentProject = oldProject;
  }
}
//---------------------------------------------------------------------windowActivated()
void StatisticalDataViewer::windowActivated(QWidget * newPlot){
  if( ! projects.size() || ! newPlot) return;
  const Project * newProject = ((Plot*)newPlot)->getProject();
  for(unsigned int i=0; i<projects.size(); i++){
    if(projects[i] == newProject){
        selectProject(i);
        break;
    }
  }
}
//---------------------------------------------------------------------makeProject()
void StatisticalDataViewer::makeProject(){ // called by newProject() and loadProject()
  projects.insert(projects.begin(), currentProject);
  graphEditor->setProject(currentProject);
  currentProject->setFocus();
  fillProjectMenu();
  connect(currentProject, SIGNAL(elementSelected(int)), graphEditor, SLOT(fillGraphView(int)));
  connect(currentProject, SIGNAL(die(Project *)), this, SLOT(closeProject(Project *)));
  connect(currentProject, SIGNAL(projectNameChanged()), this, SLOT(fillProjectMenu()));
}
//---------------------------------------------------------------------saveProject()
void StatisticalDataViewer::saveProject(){
  if(currentProject){
    currentProject->save();
  }
}
//---------------------------------------------------------------------closeProject()
void StatisticalDataViewer::closeProject(Project * deadProject){
  if( ! projects.size()) return;
  if(    ( ! deadProject && ! currentProject )
      || QMessageBox::Cancel == QMessageBox::warning( this, QString::null,
                   "Close " + (   deadProject
                                ? deadProject->getElementName()
                                : currentProject->getElementName()
                               )
                            + " ?",
                   QMessageBox::Ok,
                   QMessageBox::Cancel)
     ) return;
  if(deadProject && currentProject != deadProject){
    for(vector<Project * >::iterator iter = projects.begin();
        iter != projects.end(); iter++){
      if( *iter == deadProject ){
        projects.erase( iter );
        delete deadProject;
        break;
      }
    }
  } else if( currentProject ){
    projects.erase(projects.begin());
    delete currentProject;
    if(projects.size()){
      currentProject = projects.front();
      graphEditor->setProject(currentProject);
    } else {
      currentProject = 0;
      graphEditor->setProject(0);
    }
    fillProjectMenu();
  }
}
//---------------------------------------------------------------------selectProject()
void StatisticalDataViewer::selectProject(int index){
  vector<Project * >::iterator projectIterator = projects.begin() + index;
  if( * projectIterator != currentProject ) {
    currentProject = * projectIterator;
    projects.erase( projectIterator );
    projects.insert( projects.begin(), currentProject );
    graphEditor->setProject( currentProject );
    currentProject->setFocus();
    fillProjectMenu();
  }
}
//---------------------------------------------------------------------savePlot()
void StatisticalDataViewer::savePlot(){
  ((Plot*)(workspace->activeWindow()))->save();
}
//---------------------------------------------------------------------addProcess()
void StatisticalDataViewer::addProcess(){
  if(currentProject){
    currentProject->addGraph(Project::PROCESS);
    currentProject->highlightElement(0);
  }
}
//---------------------------------------------------------------------addPolynomial()
void StatisticalDataViewer::addPolynomial(){
  if(currentProject){
    currentProject->addGraph(Project::POLYNOMIAL);
    currentProject->highlightElement(0);
  }
}
// //---------------------------------------------------------------------addNeural
// void StatisticalDataViewer::addNeural(){
//   if(currentProject){
//     currentProject->addGraph(Project::NEURAL);
//     currentProject->highlightElement(0);
//   }
// }
//---------------------------------------------------------------------addSample()
void StatisticalDataViewer::addSample(){
  if(currentProject){
    currentProject->addGraph(Project::SAMPLE);
    currentProject->highlightElement(0);
  }
}
//---------------------------------------------------------------------addSplit()
void StatisticalDataViewer::addSplit(){
  if(currentProject){
    currentProject->addGraph(Project::SPLIT);
    currentProject->highlightElement(0);
  }
}
//---------------------------------------------------------------------toggleDoubleBuffer()
void StatisticalDataViewer::toggleDoubleBuffer(){
  bool s = ( viewMenu->isItemChecked(dblBufferID) );
  viewMenu->setItemChecked(dblBufferID, ! s );
  if(currentProject){
    currentProject->setDoubleBuffering( s );
  }
}
//---------------------------------------------------------------------toggleAxes()
void StatisticalDataViewer::toggleAnimated(){
  bool animated = !viewMenu->isItemChecked(animatedID);
  viewMenu->setItemChecked(animatedID,animated);
  if(currentProject){
    currentProject->setAnimated(animated);
  }
}
//---------------------------------------------------------------------help()
void StatisticalDataViewer::help()
{
  QMessageBox::about( this,  "Statistical Data Viewer : about",
           "<h3>Statistical Data Viewer,<br>version 1</h3>"
           "A tool for the experimental verification "
           "of the Minimum Description Length theory for "
           "data prediction. Find out more at "
           "<a href=\"http://volker.nannen.com/mdl/\">"
           "volker.nannen.com/mdl<\a>");
}
//---------------------------------------------------------------------test()
//void StatisticalDataViewer::test(){
//}
//--------------------------------------------------------------------tile()
void StatisticalDataViewer::tile(){
  workspace->tile();
  for(unsigned int i=0; i<projects.size(); i++){
    projects[i]->getCanvas()->refresh();
  }
}
//---------------------------------------------------------------------cascade()
void StatisticalDataViewer::cascade(){
  workspace->cascade();
}
//---------------------------------------------------------------------fillProjectMenu()
void StatisticalDataViewer::fillProjectMenu(){
  projectMenu->clear();
  for(unsigned int i=0; i < projects.size(); i++){
    int ID = projectMenu->insertItem(projects[i]->getElementName(), this, SLOT(selectProject(int)) );
    projectMenu->setItemParameter(ID, i);
  }
}

