/***************************************************************************
                          distribution.cpp  -  description
                             -------------------
    begin                : Wed Dec 18 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "distribution.h"

#include <cmath>
#include <cstdlib>

using namespace std;

Distribution::Distribution(const int ID, QDomElement * /*element*/,
                           const QString elementName)
  : BasicElement(ID, elementName)
{
  hideName();
}
Distribution::~Distribution(){;}

void Distribution::xml_list(){
  BasicElement::xml_list();
}
void Distribution::setParameters(double /*parameter1*/, 
                                 double /*parameter2*/, 
                                 double /*parameter3*/)
{;}
double Distribution::draw(){
  return 0;
}

//-------------------------------------------------class UniformDistribution

UniformDistribution::UniformDistribution(const int ID, QDomElement * element)
  : Distribution(ID, element, "uniform distribution")
{
  xml_read(element);
}
void UniformDistribution::xml_list(){
  Distribution::xml_list();
  xml_double("lower limit", lowerLimit, -1, EDIT);
  xml_double("upper limit", upperLimit, 1,  EDIT);
}
void UniformDistribution::setParameters(double newLowerLimit, 
                                        double newUpperLimit, 
                                        double /*parameter3*/){
  lowerLimit = newLowerLimit;
  upperLimit = newUpperLimit;
}
double UniformDistribution::draw(){
  return ( rand() * div * (upperLimit - lowerLimit) ) + lowerLimit;;
}
double UniformDistribution::div(1.0 / RAND_MAX);

//-------------------------------------------------class GaussianDistribution

GaussianDistribution::GaussianDistribution(const int ID, QDomElement * element)
  : Distribution(ID, element, "gaussian distribution")
{
  xml_read(element);
}
void GaussianDistribution::xml_list(){
  Distribution::xml_list();
  xml_double("mean", mean, 0, EDIT);
  xml_double("variance", variance, 1, EDIT);
}
void GaussianDistribution::setParameters(double newVariance, 
                                         double newMean, 
                                         double /*parameter3*/){
  variance = newVariance;
  mean = newMean;
}
double GaussianDistribution::draw(){
  static bool second(false);
  static double y2;
  if(second){
    second = false;
    return variance * y2 + mean;
  } else {
    second = true;
    float x1, x2, w;
    do {
      x1 = rand() * div - 1.0;
      x2 = rand() * div - 1.0;
      w = x1 * x1 + x2 * x2;
    } while ( w >= 1.0 );
    w = sqrt( (-2.0 * log( w ) ) / w );
    y2 = x2 * w;
    return variance * x1 * w + mean;
  }
}
double GaussianDistribution::div(2.0 / RAND_MAX);

//-------------------------------------------------class ExponentialDistribution

ExponentialDistribution::ExponentialDistribution(const int ID, QDomElement * element)
  : Distribution(ID, element, "exponential distribution")
{
  xml_read(element);
}
void ExponentialDistribution::xml_list(){
  Distribution::xml_list();
  xml_double("mean", mean, 0, EDIT);
  xml_double("lambda", lambda, 1, EDIT);
}
void ExponentialDistribution::setParameters(double newLambda, 
                                            double newMean, 
                                            double /*parameter3*/){
  lambda = newLambda;
  mean = newMean;
}
double ExponentialDistribution::draw(){
  double result;
  bool negative = true;
  do {
    result = rand() * div;
    if( result > 1 ) {
      negative = false;
      result -= 1;
    }
  } while ( ! result );
  result = log( result );
  if ( lambda ) result /= lambda;
  if ( negative ) result = -result;
  return result + mean;
}
double ExponentialDistribution::div(2.0 / RAND_MAX);

//-------------------------------------------------class UniformDistribution

CauchyDistribution::CauchyDistribution(const int ID, QDomElement * element)
  : Distribution(ID, element, "cauchy distribution")
{
  xml_read(element);
}
void CauchyDistribution::xml_list(){
  Distribution::xml_list();
  xml_double("mean", mean, 0, EDIT);
  xml_double("scale", scale, 0.1, EDIT);
}
void CauchyDistribution::setParameters(double newScale, 
                                       double newMean, 
                                       double /*parameter3*/){
  scale = newScale;
  mean = newMean;
}
double CauchyDistribution::draw(){
  return scale * tan ( rand() * div - 1.570796327 ) + mean;
}
double CauchyDistribution::div(3.14159265 / RAND_MAX);


