/***************************************************************************
                          distribution.h  -  description
                             -------------------
    begin                : Wed Dec 18 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DISTRIBUTION_H
#define DISTRIBUTION_H

#include "basicelement.h"

class QDomElement;

class Distribution : public BasicElement  {
public: 
  Distribution(const int ID=0, QDomElement * element=0, const QString elementName=0);
  ~Distribution();
  virtual const QString tagName() const { return "Distribution"; }

  
  virtual void xml_list();
  virtual void setParameters(double parameter1=0, double parameter2=0, double parameter3=0);
  virtual double draw();
};

//-------------------------------------------------class UniformDistribution

class UniformDistribution: public Distribution {
public:
  UniformDistribution(const int ID=0, QDomElement * element = 0);

public:
  virtual void xml_list();
  virtual void setParameters(double newLowerLimit=-1, double newUpperLimit=1, double parameter3=0);
  virtual double draw();

private:
  double lowerLimit;
  double upperLimit;
  static double div;
};

//-------------------------------------------------class GaussianDistribution

class GaussianDistribution: public Distribution {
public:
  GaussianDistribution(const int ID=0, QDomElement * element = 0);

public:
  virtual void xml_list();
  virtual void setParameters(double newVariance=1, double newMean=0, double parameter3=0);
  virtual double draw();

private:
  double variance;
  double mean;
  static double div;
};

//-------------------------------------------------class ExponentialDistribution

class ExponentialDistribution: public Distribution {
public:
  ExponentialDistribution(const int ID=0, QDomElement * element = 0);

public:
  virtual void xml_list();
  virtual void setParameters(double newLambda=1, double newMean=0, double parameter3=0);
  virtual double draw();

private:
  double lambda;
  double mean;
  static double div;
};

//-------------------------------------------------class ExponentialDistribution

class CauchyDistribution: public Distribution {
public:
  CauchyDistribution(const int ID=0, QDomElement * element = 0);

public:
  virtual void xml_list();
  virtual void setParameters(double newScale=1, double newMean=0, double parameter3=0);
  virtual double draw();

private:
  double scale;
  double mean;
  static double div;
};

#endif
