/***************************************************************************
                          domenumeration.cpp  -  description
                             -------------------
    begin                : Mon Apr 29 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "domenumeration.h"
#include "basicelement.h"
#include "collection.h"

#include <iostream>

using namespace std;

DomEnumeration::DomEnumeration(QString name)
  : name(name),
    collection(0)
{}
DomEnumeration::~DomEnumeration(){
}
void DomEnumeration::init(Collection * newCollection,
                          const QString newCriterion){
  clear();
  add(newCollection, newCriterion);
}
void DomEnumeration::add(Collection * newCollection,
                         const QString newCriterion){
  collection = newCollection;
  criterion = newCriterion;
  if( ! collection ) return;
  for(Collection::iterator iter = collection->begin(); iter != collection->end();
      iter++){
    if( (*iter)->tagName() == criterion || criterion.isNull()){
      insert((*iter)->getElementName(), (*iter)->getID());
    }
  }
}
bool DomEnumeration::insert(const QString newString, const int value){
  if(value < 0) return false;
  if(contents.size()){
    for(vector<DomEnumerationItem>::iterator iter = contents.begin();
        iter != contents.end(); iter++){
      if(iter->value == value) return false;
    }
  }
  contents.push_back(DomEnumerationItem(newString, value));
  return true;
}
QString DomEnumeration::stringAtValue(const int & value){
  if(contents.size()){
    for(vector<DomEnumerationItem>::iterator iter = contents.begin();
        iter != contents.end(); iter++){
      if(iter->value == value){
        return iter->string;
      }
    }
  }
  return QString::null;
}
int DomEnumeration::valueAtString(const QString & string){
  if(contents.size()){
    for(vector<DomEnumerationItem>::iterator iter = contents.begin();
        iter != contents.end(); iter++){
      if(iter->string == string)
        return iter->value;
    }
  }
  return -1;
}
int DomEnumeration::indexOfValue(const int & value) const{
  for(unsigned int position=0; position < contents.size(); position++){
    if(contents[position].value == value)
      return position;
  }
  return -1;
}
int DomEnumeration::indexOfString(const QString & string) const{
  for(unsigned int position=0; position < contents.size(); position++){
    if(contents[position].string == string)
      return position;
  }
  return -1;
}
QString DomEnumeration::stringAtIndex(const int & index) const{
  if(index >= 0 && index < (int)contents.size() ){
    return contents[index].string;
  }
  return QString::null;
}
int DomEnumeration::valueAtIndex(const int & index) const{
  if(index >= 0 && index < (int)contents.size()){
    return contents[index].value;
  }
  return -1;
}
BasicElement * DomEnumeration::elementAtIndex(const int & index) const{
  if(index >= 0 && index < (int)contents.size() ){
    return collection->IDToElement(contents[index].value);
  }
  return (BasicElement *)(0);
}
BasicElement * DomEnumeration::elementAtValue(const int & value) const{
  int index = indexOfValue(value);
  if(index >= 0){
    return collection->IDToElement(contents[index].value);
  } else {
    return (BasicElement *)(0);
  }
}
void DomEnumeration::removeID(const int ID){
  if(contents.size()){
    for(vector<DomEnumerationItem>::iterator iter = contents.begin();
        iter != contents.end(); iter++){
      if(iter->value == ID){
        contents.erase(iter);
        return;
      }
    }
  }
}
//------------------------------DomEnumerationItem

DomEnumerationItem::DomEnumerationItem(QString string, int value)
  : string(string),
    value(value)
{
}
