/***************************************************************************
                          graph.h  -  description
                             -------------------
    begin                : Mon Mar 11 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GRAPH_H
#define GRAPH_H

#include "basicelement.h"
#include "points.h"

class Project;
class QPainter;
class QDomElement;
class Project;
class LaTeX;

class Graph : public BasicElement {
public:
  Graph(Project * project, const QString name="graph");
  Graph(Graph * other);
  virtual ~Graph();
  virtual const QString tagName() const { return "BasicGraph"; }

  virtual bool xml_read(QDomElement * element);
  virtual void xml_list();
  virtual void setValid(const bool yes=true);
  virtual void setID(const int newID);
  virtual void destroy();
  virtual void refresh();
  virtual void toggleColor(const bool yes=true);
  virtual void setPen(const QPen & newPen)           { points.setPen(newPen); }
  virtual Graph * copyGraph()                        { return 0;}

  void setDefaultColor(const QString color)          { points.setDefaultColor(color); }

public: // virtual form this level onwards
  virtual void recalculate()                         { valid = true; }
  virtual void setVisible(const bool yes=true);
  void statistics();
  double getSigma()                                  { return sigma; }

  Points * getPoints()                               { return &points; }

protected:
  void updateCanvas();
  ScientificCanvas * makePlot(Scale::Method scaleMethod=Scale::SIMPLE,
                           double lowerLimit=0, double upperLimit=250,
                           double leftLimit=0,  double rightLimit=100  );

protected:
  Project * project;
  bool copyBool, controlCopyBool, canCopy;
  bool deleteBool;
  bool toggled;
  Points points;
  double mean, sigma, sum, sumOfSquares;
  int pointCount;
  int plotID;
};

#endif
