/***************************************************************************
                          grapheditor.cpp  -  description
                             -------------------
    begin                : Thu Dec 19 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "grapheditor.h"
#include "attribute.h"
#include "grapheditview.h"
#include "graph.h"
#include "collection.h"

#include <qlistbox.h>
#include <qdom.h>

GraphEditor::GraphEditor(QWidget * parent, const char * name)
  : QSplitter(Qt::Vertical, parent, name),
    project(0)
{
  graphList = new QListBox(this, "graph selection list");
  graphList->viewport()->setBackgroundMode(Qt::PaletteBase, Qt::PaletteBackground);
  graphList->setBaseSize(graphList->baseSize().width(), 100);
  graphList->resize(graphList->size().width(), 100);
  graphList->viewport()->setBackgroundMode(Qt::PaletteBase);
  graphView = new GraphEditView(this, "graph manipulation table");
  graphView->setColumnWidth( graphView->addColumn("attribute"), 170 );
  graphView->setColumnWidth(graphView->addColumn("value"), 120);
  graphView->setSorting(-1,true);
  graphView->setAllColumnsShowFocus(true);
  graphView->setTreeStepSize(0);
  graphView->setItemMargin(EDITOR_ATTRIBUTE_MARGIN);
  graphView->viewport()->setBackgroundMode(Qt::PaletteBackground);
  connect(graphList, SIGNAL(selected(int)), this, SLOT(elementSelected(int)));
  connect(graphList, SIGNAL(highlighted(int)), this, SLOT(elementHighlighted(int)));
  setResizeMode(graphList, QSplitter::Stretch);
  setResizeMode(graphView, QSplitter::Stretch);
}
void GraphEditor::resizeEvent( QResizeEvent * e ){
  QSplitter::resizeEvent(e);
  graphView->setColumnWidth(1,graphView->visibleWidth() - graphView->columnWidth(0) );
}
void GraphEditor::setProject(Project * newProject){
  project = newProject;
  graphView->setProject(project);
  if(project){
    fillListBox();
    fillGraphView( project->getSelectedElement() );
    connect(project, SIGNAL(fillListBox()), this, SLOT(fillListBox()));
//    connect(project, SIGNAL(elementHighlighted(int)), this, SLOT(highlight(int)));
  } else {
    graphList->clear();
    graphView->clear();
  }
}
//void GraphEditor::highlight(int index){
//  graphList->setCurrentItem( graphs()->IDToIndex( index  ) + 1 );
//}
void GraphEditor::fillListBox(){
  graphList->clear();
  if(project){
    graphList->insertItem(project->getElementName());
    if(graphs()->size()){
      for(Collection::iterator iter = graphs()->begin(); iter != graphs()->end(); iter++){
        graphList->insertItem((*iter)->getElementName());
      }
    }
  }

}
void GraphEditor::elementSelected(int index){
  fillGraphView( graphs()->IndexToID(index-1) );
}
void GraphEditor::elementHighlighted(int index){
  if( project ) project->highlightElement( graphs()->IndexToID(index-1) );
}
void GraphEditor::fillGraphView(const int ID){
//  cerr << "GraphEditor::fillGraphView, ID=" << ID << " \n";
  if( project ){
    BasicElement * element = graphs()->IDToElement(ID);
    if( element ){
      element->xml_edit(graphView);
    } else {
      project->xml_edit(graphView);
    }
  }
  graphView->setColumnWidth(1,graphView->visibleWidth() - graphView->columnWidth(0) );
  graphList->setCurrentItem( graphs()->IDToIndex( ID  ) + 1 );
  elementHighlighted(0);
  if( project ) project->selectElement( ID );
}
/*
void GraphEditor::testDom(QDomElement * element, int indent){
  QDomNamedNodeMap map = element->attributes();
  for(int i=0; i < indent; i++){ cerr << "  "; }
  cerr << element->tagName() << " size: " << map.length() << endl;
  for (int index=0; index < map.length(); index++){
    for(int i=0; i < indent; i++){ cerr << "  "; }
    cerr << "  attribute " << map.item(index).nodeName() << endl;
  }

  cerr << endl;
  for(int i=0; i < indent; i++){ cerr << "  "; }
  cerr << "  children: \n" << endl;

  QDomNode n = element->firstChild();
  while( !n.isNull() ) {
      QDomElement e = n.toElement(); // try to convert the node to an element.
      if( !e.isNull() ) { // the node was really an element.
          for(int i=0; i < indent + 1; i++){ cerr << "--"; }
          cerr << "tagName: " << e.tagName() << endl;
          testDom(&e, indent + 2);
      }
      n = n.nextSibling();
  }
}*/
