/***************************************************************************
                          matrix.h  -  description
                             -------------------
    begin                : ?' ??? 5 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MATRIX_H
#define MATRIX_H

#include "basicelement.h"

#include <m_apm.h>

#include <qstring.h>

//#define MAPM double
//#define MM_Zero 0

class Points;
class EnhancedProgressBar;

class Matrix : public BasicElement {
public:
  Matrix(const QString name);
  Matrix(const int x = 1, const int y = 1);
  Matrix(Matrix & other);
  ~Matrix();
  virtual const QString tagName() const { return "Matrix"; }

  void init(int x = 1, int y = 1);
  void resize(int x = 1, int y = 1);

  virtual bool xml_read(QDomElement * element);
  virtual bool xml_write(QDomElement * parent);
  virtual void xml_list();

  MAPM & operator[](const int & index)            { return array[index]; }
  MAPM & operator()(const int & x, const int & y) { return matrix(x,y); }
  Matrix & operator+=(Matrix & other);
  MAPM & matrix(const int & x, const int & y)     { return array[x * columns + y]; }
  void swapRows(const int & a, const int & b);
  void swap(const int & a, const int & b);
  void setPrecision(int & newPrecision)           { precision = newPrecision; }
  int getPrecision()                              { return precision; }
  void makeZero();
  bool makeTriangular(EnhancedProgressBar * dialog=0);
  double determinant(int degree);
  double logDeterminant(int degree, bool & valid);
  const int getSize() const                       { return size; }
  const int getColumns() const                    { return columns; }
  const int getRows() const                       { return rows; }
  QString toString(const int index );
  static double toDouble(const MAPM & value);

private:
  void destroy();

private:
  MAPM * array;
  int size, rows, columns;
  int precision;
};

#endif
