/***************************************************************************
                          parsenode.cpp  -  description
                             -------------------
    begin                : ?' ??? 29 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "parsenode.h"

#include <qregexp.h>

#include <cmath>
#include <iostream>



#include <qstring.h>

#include <dlfcn.h>

#include <cstdio>
#include <cctype>
#include <cstdlib>
#include <string>

using namespace std;

VALUETYPE ParseNode::pi = VALUETYPE(3.141592654);
VALUETYPE ParseNode::expo = VALUETYPE(exp(1.0));
VALUETYPE ParseNode::log_constant = VALUETYPE(0.5 * log(exp(1.0) * 3.141592654 * 2));

void ParseNode::parse(QString expression){
  destroy();
//  cerr << " expr " << expression << endl;
  expression.replace(QRegExp("\\s"), " ");
  const char * ptr = expression.latin1();
  if( ! ptr ) return;
  args0 = parseExpression(&ptr);
}
void ParseNode::destroy(){
  if(args0) { delete args0; args0=0; }
  if(args1) { delete args1; args1=0; }
}
double ParseNode::evalFunction(const char *name, double arg) {
  static void *lib = dlopen(NULL, RTLD_LAZY);
  double (*func)(double x);
  func = (double (*)(double))dlsym(lib, name);
  return func ? func(arg) : 0;
}

VALUETYPE ParseNode::eval(const int c, const int k, const int n, const VALUETYPE s){
  switch(op) {
    case BASE: return args0 ? args0->eval(c, k, n, s) : 0;
    case CVAR: return c;
    case DVAR: return k;
    case NVAR: return n;
    case SVAR: return s;
    case LOGSVAR: return 0.5 * log( s ) + log_constant + 10;
    case EXPO: return expo;
    case PI: return pi;
    case NEGATE: return -args0->eval(c, k, n, s);
    case POWER: return pow(args0->eval(c, k, n, s), args1->eval(c, k, n, s));
    case DIVIDE: {
      VALUETYPE divisor = args1->eval(c, k, n, s);
      return (divisor ? (args0->eval(c, k, n, s) / divisor) : 0);
    }
    case MULTIPLY: return args0->eval(c, k, n, s) * args1->eval(c, k, n, s);
    case ADD: return args0->eval(c, k, n, s) + args1->eval(c, k, n, s);
    case SUBTRACT: return args0->eval(c, k, n, s) - args1->eval(c, k, n, s);
    case NUM: return val;
    case UNARYFUNC:  return evalFunction(funcName.latin1(),
                                         args0->eval(c, k, n, s));
  }
  return 0;
}

ParseNode * ParseNode::parseExpression(const char **ptr, int oldPre) {
//  cerr << " oldPre: " << oldPre << " \t parseExpression " << *ptr << endl;
  ParseNode * nextTerm = 0;
  ParseNode * result = parseValue(ptr);
  if( ! result ) { parseError(*ptr); return 0; }
  while( **ptr == ' ') (*ptr)++;
  char c = **ptr;
  int pre = precedence(c);
  while(pre > oldPre){
    (*ptr)++;
    nextTerm = parseExpression(ptr, pre);
    if( ! nextTerm ) { return 0; }
    result = new ParseNode(c, result, nextTerm);
    while( **ptr == ' ') (*ptr)++;
    c = ** ptr;
    pre = precedence(c);
  }
  return result;
}
ParseNode * ParseNode::parseValue(const char **ptr) { // four cases: number, variable,
                                                // unary neg, or parensExpression
  ParseNode * result = 0;
  bool negate = false;
  char c = **ptr;
  while (c == '-' || c == ' ') {
    if( c == '-' ) negate = negate ? false : true;
    (*ptr)++;
    c = **ptr;
  }
  if (c == '(') {
    (*ptr)++;
    result = parseExpression(ptr);
    while( **ptr == ' ') (*ptr)++;
    if (**ptr != ')') {
      parseError(*ptr);
      return 0;
    }
    (*ptr)++;
    return result;
  }
  if ( c == 0) return 0;
  int length = strspn(*ptr, "0123456789.");
  if( length ){
    char fbuf[length + 1];
//    cerr << "parseValue::num, c: " << length << "\n";
    strncpy(fbuf, *ptr, length);
    (*ptr) += length;
    fbuf[length] = 0;
    result = new ParseNode(NUM, atof(fbuf));
  } else {
    length = strspn(*ptr, "0123456789abcdefghijklmnopqrstuvwxyzS");
    if( length == 1 && (    c == CVAR || c == DVAR    || c == NVAR 
                         || c == SVAR || c == LOGSVAR || c == EXPO
                         || c == PI   ) ) {
      (*ptr)++;
      result = new ParseNode(c, 0, 0);
      return negate ? new ParseNode(NEGATE, result, 0) : result;
    } else if( length ) {
      char fbuf[length + 1];
      strncpy(fbuf, *ptr, length);
      (*ptr) += length;
      fbuf[length] = 0;
      ParseNode * nextTerm = parseValue(ptr);
      if( nextTerm) result = new ParseNode(UNARYFUNC, nextTerm, 0, fbuf);
    }
  }
  if( result && negate ) result = new ParseNode(NEGATE, result, 0);
  return result;
}
int ParseNode::precedence(char sign){
  switch(sign){
    case '-': return 1;
    case '+': return 2;
    case '*': return 3;
    case '/': return 3;
    case '^': return 5;
  }
  return 0;
}
void ParseNode::parseError(QString error){
  cerr << "Sorry, parse value error for " << error << endl;
}
