/***************************************************************************
                          parsenode.h  -  description
                             -------------------
    begin                : ?' ??? 29 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PARSENODE_H
#define PARSENODE_H

/*
parser contribution
Date: Wed, 23 Oct 2002 11:25:17 +0200
From: Rudi Cilibrasi <Rudi.Cilibrasi@cwi.nl>
To: Volker Nannen <Volker.Nannen@cwi.nl>

Hi Volker,

I was bored and threw together a parser suitable for use in your
program. It supports the standard unary math library functions,
the four functions, and exponentiation using ^. I have included
the makefile and .c below. Please let me know if you have any
questions on how to use it...

Rudi

Makefile:
parse: parse.cc
gcc parse.cc -o parse -lm -g -ldl

*/

//parse.cc:

#include <qstring.h>

using namespace std;

#define BASE 'b'
#define MULTIPLY '*'
#define NEGATE '~'
#define POWER '^'
#define DIVIDE '/'
#define ADD '+'
#define SUBTRACT '-'
//#define XVAR 'x'
#define NVAR 'n'
#define DVAR 'k'
#define CVAR 'c'
#define SVAR 's'
#define LOGSVAR 'S'
#define EXPO 'e'
#define PI 'p'

#define NUM '#'
#define UNARYFUNC '['

#define MAXLEN 128

typedef double VALUETYPE;

class ParseNode {

private:
  char op;
  ParseNode * args0, * args1;
  VALUETYPE val;
  QString funcName;

public:
  ParseNode()
    : op(BASE), args0(0), args1(0) {}
  ParseNode(char op, VALUETYPE val)
    : op(op), args0(0), args1(0), val(val) {}
  ParseNode(char op, char * str)
    : op(op), args0(0), args1(0), funcName(str) {}
  ParseNode(char op, ParseNode *arg0, ParseNode *arg1, char * str=0)
    : op(op), args0(arg0), args1(arg1), funcName(str) {}

  ~ParseNode(){ destroy(); }

  void parse(QString expression);
  VALUETYPE eval(const int c, const int k, const int n, const VALUETYPE s);

private:
  void destroy();
  ParseNode *parseValue(const char **ptr);
  ParseNode *parseExpression(const char **ptr, int oldPre=0);
  double evalFunction(const char *name, double arg);
  void parseError(QString error);
  int precedence(char sign);
  static VALUETYPE expo;
  static VALUETYPE pi;
  static VALUETYPE log_constant;

};

#endif
