/***************************************************************************
                          plot.h  -  description
                             -------------------
    begin                : Fri Mar 1 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PLOT_H
#define PLOT_H

#include <qcanvas.h>

class Project;
class Axes;
class SDVStatusBar;
class ScientificCanvas;

class QString;
class QMouseEvent;
class QLabel;
class ScalePlot;
class QCloseEvent;
class Project;
class QResizeEvent;

class Plot : public QCanvasView {

public:
  ~Plot();
  Plot(Project * project, ScientificCanvas * canvas, const char *name=0, WFlags f=0);
  void save();
  const Project * getProject()                          { return project; }
  void setCanvas(ScientificCanvas * newCanvas);
  virtual void resizeContents(int w, int h);
  virtual void closeEvent(QCloseEvent * event);

protected:
  void contentsMousePressEvent(QMouseEvent * event);
  void contentsMouseDoubleClickEvent(QMouseEvent * event);
  void contentsMouseMoveEvent(QMouseEvent * event);
  void contentsMouseReleaseEvent(QMouseEvent * event);
  void mouseReleaseEvent(QMouseEvent * event);
  void leaveEvent(QEvent * event);
  void enterEvent(QEvent * event);
  virtual void viewportResizeEvent(QResizeEvent * event);
  double getCurrentX()                { return currentX; }
  double getCurrentY()                { return currentY; }

private:
  void destroy();

private:
  ScientificCanvas * scientificCanvas;
  SDVStatusBar * statusBar;
  QLabel * labelX, * labelY;
  Axes * axes;
  ScalePlot * scalePlot;
  Project * project;
  bool scaling;
  bool resized;
  int scaleSize;
  double scale;
  double currentX, currentY;
};

//------------------------------------------------------class ScalePlot

class ScalePlot : public QCanvasView {
public:
  ScalePlot::ScalePlot(QWidget *parent, double scale, int size, const char *name=0);

  virtual void drawContents( QPainter * p, int cx, int cy, int cw, int ch );
  virtual void move(int x, int y, int contX, int contY);
  virtual void resizeContents(int w, int h);
//  virtual void paintEvent

  void setScale( int newScale )       { scale = newScale; }
  void setScaleSize( int newSize )    { scaleSize = newSize; }

private:
  double scaleX, scaleY;
  QWidget * parent;
  double scale;
  int scaleSize;
  int offset;
};

#endif
