/***************************************************************************
                          points.h  -  description
                             -------------------
    begin                : Thu May 9 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef POINTS_H
#define POINTS_H

#include "basicelement.h"
#include "singlepoint.h"

#include <qpen.h>

#include <vector>

using namespace std;

class QDomElement;
class Axes;
class ScientificCanvas;
class EnhancedProgressBar;

class Points : public vector<SinglePoint * >, public BasicElement {
public:
  Points(Points & points, const int ID=0, ScientificCanvas * canvas=0);
  Points(const int ID=0, ScientificCanvas * canvas=0);
  ~Points();
  virtual const QString tagName() const { return "Graph"; }
  bool copy(Points & other, EnhancedProgressBar * dialog=0);
  virtual bool xml_read(QDomElement * element, EnhancedProgressBar * dialog=0);
  virtual bool xml_write(QDomElement * parent);
  virtual void xml_list();
  void push_back(SinglePoint * point);
  void push_back(QDomElement * element=0)
    { push_back(new SinglePoint(getID(), canvas, element)); }
  void push_back(const int posX, const int posY, const bool & valid=true)
    { push_back(new SinglePoint(getID(), canvas, posX, posY, thickness, valid)); }
  void push_back(const double & valX, const double & valY, const bool & valid=true)
    { push_back(new SinglePoint(getID(), canvas, valX, valY, thickness, valid)); }
  void paint_back(const double & valX, const double & valY, const bool & valid=true);
  void push_back(const int posX, const int posY, const double & valX, const double & valY, const bool & valid=true)
    { push_back(new SinglePoint(getID(), canvas, posX, posY, valX, valY, thickness, valid)); }
  void copy(SinglePoint * otherPoint, const bool & valid=true)
    { push_back(new SinglePoint(getID(), canvas, otherPoint, thickness, valid)); }
  void setContinuous(bool yes = true, EnhancedProgressBar * dialog=0);
  void setVisible(const bool yes=true);
  bool getVisible()                                    { return visible; }
  void clear();              // clear deletes the points
  void setPen(const QPen & newPen);
  int recalculate(EnhancedProgressBar * dialog=0);
  void setCanvas(ScientificCanvas * newCanvas=0);
  void setID(const int newID);
  void bounds(double & up, double & down, double & left, double & right);
  void setDefaultColor(const QString newColor);
  void setColor(const QString newColor);
  QString getColor()                               { return color; }
  void refresh();
  void setThick(const bool yes=true);
  void makeControl();
//  void setPreserve(const bool yes=true)                { preserve = yes; }
  void destroy();

private:
  ScientificCanvas * canvas;
  bool continuous, controlContinuous;
  bool visible, controlVisible;
  bool thick, controlThick;
  int thickness;
  int outside;
  QString color, controlColor, defaultColor;
  QPen pen;
};

#endif
