/***************************************************************************
                          polynomial.h  -  description
                             -------------------
    begin                : Thu May 2 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef POLYNOMIAL_H
#define POLYNOMIAL_H

#include "graph.h"
#include "matrix.h"
#include "scale.h"
#include "domenumeration.h"

class Project;
class Sample;
class Collection;
class Points;
class ParseNode;

class Polynomial : public Graph {

public:
  Polynomial(Project * project, const QString name="polynomial", QDomElement * element=0);
  Polynomial(Polynomial * other);
  ~Polynomial();
  virtual const QString tagName() const { return "Polynomial"; }

public:
  virtual bool xml_read(QDomElement * element);
  virtual void xml_list();
  virtual void refresh();
  virtual void recalculate();
  virtual void setValid(const bool yes);
  virtual void setID(const int ID=0);
  double getY(const double x);
  int getMaxOrder()                            { return maxOrder; }
  void setPrecision(int max, const int * array);
  QString getParameter(const int index)        {
    return (index >= 0) && (index <= maxOrder) ?
    parameter.toString(index) : QString::null; }
  int getPrecision(const int index)            {
    return (index >= 0) && (index <= maxOrder) ?
    precisionArray[index] : 0; }

private:
  void init();
  void destroy();
  void makeControl();
  double evaluate(const double & x);
  void setPrecision(const int newPrecision);
  void setMaxOrder(int newMaxOrder);
  int getComplexity();
//  void makeMethod(QDomElement * element=0);

protected:
  void setOrder(const int newOrder, const int newPrecision=-1);
  bool makeParameter();

  void testGoodness();
  int goodnessOfFit(double & myVariance, Sample * sample=0);
  Sample * getSample(const int ID);

protected:
  enum Method {
    MANUAL,
    CROSS,
    TWO_PART_MDL,
    TRIMMED_MDL,
    MIXTURE_MDL,
    ANALYSIS,
    COMPLEX,
    ORDER_PRECISION
  };
  
  class MethodEnumeration : public DomEnumeration{
    public: MethodEnumeration();
  };
  static MethodEnumeration methodEnumeration;

protected:
  int method, controlMethod;
  DomEnumeration * graphEnumeration;
  int learnSampleID, controlLearnSampleID, testSampleID, controlTestSampleID;
  QString penalty, controlPenalty;
  ParseNode * penaltyTerm;
  int order, controlOrder;
  bool fit, controlFit;
  bool editPrecision;
  int maxOrder, controlMaxOrder;
  int precision, controlPrecision;
  double variance, testVariance;
  int descriptionLength;
  int stepNumber, sampleNumber;
  Matrix parameter;
  int * precisionArray;
};


#endif
