/***************************************************************************
                          precisioneditor.cpp  -  description
                             -------------------
    begin                : ?' ??? 16 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "precisioneditor.h"
#include "polynomial.h"

#include <qhbox.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <qpushbutton.h>
#include <qscrollview.h>

PrecisionEditor::PrecisionEditor(Polynomial * polynomial)
  : polynomial(polynomial)
{
  setCaption("SDV precision editor");
  max = polynomial->getMaxOrder();
  array = new int[max+1];
  backupArray = new int[max+1];
  label = new (QLabel*)[max+1];
  input = new (QLineEdit*)[max+1];
  for(int i=0; i<=max; i++) backupArray[i] = polynomial->getPrecision(i);
  buildDialog();
}
PrecisionEditor::~PrecisionEditor(){
  delete[] array; array = 0;
  delete[] backupArray; backupArray = 0;
  delete[] label; label = 0;
  delete[] input; input = 0;
}
void PrecisionEditor::buildDialog(){
  setFixedSize(400,620);
  QSizePolicy sizePolicy;
//  sizePolicy = this->sizePolicy();
//  sizePolicy.setHorData(QSizePolicy::Expanding);
//  this->setSizePolicy(sizePolicy);

  QVBox * superBox = new QVBox(this);
  superBox->setMinimumSize(400,600);
  superBox->setSpacing(20);

  setSizeGripEnabled(true);
  QScrollView * scroll = new QScrollView( superBox );
  QVBox * mainBox = new QVBox(scroll->viewport());
  mainBox->setMinimumSize(780,580);
  sizePolicy = mainBox->sizePolicy();
  sizePolicy.setHorData(QSizePolicy::Expanding);
  mainBox->setSizePolicy(sizePolicy);
  scroll->addChild(mainBox);
  mainBox->setMargin(10);

  QIntValidator * validator = new QIntValidator(0, max, this );

  for(int i=0; i<=max; i++){
    QHBox * box = new QHBox(mainBox);
    box->setSpacing(20);
    QLabel * degree = new QLabel(QString::number(i)+")", box);
    degree->setFixedWidth(30);
    input[i] = new QLineEdit(QString::number(current>=i ? precision : 0), box);
    input[i]->setValidator(validator);
    input[i]->setFixedWidth(40);
    label[i] = new QLabel(box);
    label[i]->setFixedWidth(600);
  }

  QHBox * box = new QHBox(mainBox);
  box->setSpacing(20);
  QLabel * dummy = new QLabel("sum", box);
  dummy->setFixedWidth(40);
  sumLabel = new QLabel("0", box);
  sumLabel->setFixedWidth(60);
  dummy = new QLabel(box);
  dummy->setFixedWidth(570);

  QHBox * buttonBox = new QHBox(superBox);
  buttonBox->setSpacing(30);
  buttonBox->setMargin(30);
  QPushButton * apply  = new QPushButton("&Apply",  buttonBox);
  QPushButton * reset  = new QPushButton("&RESET",     buttonBox);
  QPushButton * close  = new QPushButton("&CLOSE",     buttonBox);

  connect(apply,  SIGNAL(clicked()), this, SLOT(apply ()));
  connect(reset,  SIGNAL(clicked()), this, SLOT(reset()));
  connect(close,  SIGNAL(clicked()), this, SLOT(accept()));

  for(int i=0; i<=max; i++){
    array[i] = backupArray[i];
    input[i]->setText(QString::number(array[i]));
  }
  writePrecision();

  exec();
}
void PrecisionEditor::apply(){
  for(int i=0; i<=max; i++){
    QString n = input[i]->text().stripWhiteSpace();
    array[i] = (n.isEmpty() ? 0 : n.toInt());
  }
  polynomial->setPrecision(max, array);
  writePrecision();
}
void PrecisionEditor::reset(){
  for(int i=0; i<=max; i++){
    array[i] = backupArray[i];
    input[i]->setText(QString::number(array[i]));
  }
  polynomial->setPrecision(max, array);
  writePrecision();
}
void PrecisionEditor::writePrecision(){
  int sum = 0;
  for(int i=0; i<=max; i++){
    label[i]->setText(polynomial->getParameter(i));
    sum += array[i];
  }
  sumLabel->setText(QString::number(sum));
}

