/***************************************************************************
                          process.h  -  description
                             -------------------
    begin                : Fri May 10 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PROCESS_H
#define PROCESS_H

#include "graph.h"
#include "domenumeration.h"

class Sample;
class Project;
class QDomElement;
class TimeSeries;

class Process : public Graph  {
public:
  Process(Project * project, const QString name="process", QDomElement * element=0);
  Process(Process * other);
  ~Process();
  virtual const QString tagName() const { return "Process"; }

public:
  virtual bool xml_read(QDomElement * element);
  virtual void xml_list();
  virtual void refresh();
  virtual void recalculate();
  virtual void setID(const int ID=0);
  virtual Graph * copyGraph()               { return new Process(this); }

  double getBegin()                     { return begin; }
  double getEnd()                       { return end; }
  double getY(const double x);

private:
  void init();
  void destroy();
  void run();
  void makeSeries(QDomElement * element=0);

private:
  enum ProcessSeries {
    NONE,
    AUTOREGRESSION,
    SINUS,
    HENON,
    LOGISTIC,
    LORENZ,
    PENDULUM,
    STEP,
    THOM
  };
  
  class SeriesEnumeration : public DomEnumeration {
    public:
      SeriesEnumeration();
  };
  static SeriesEnumeration seriesEnumeration;

private:
  int series, controlSeries;
  TimeSeries * timeSeries;            // when changing, change processEnumeration too!
  double begin, end, controlBegin, controlEnd;
  bool make, controlMake;
  double step, invStep;
  int seed;
};

#endif
