/***************************************************************************
                          project.h  -  description
                             -------------------
    begin                : Tue Mar 19 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PROJECT_H
#define PROJECT_H

#include "basicelement.h"
#include "scientificcanvas.h"
#include "collection.h"

class Workspace;
class ScientificCanvas;
class WorkBench;
class Graph;

class QWidget;
class QDomElement;
class QFile;

class Project : public BasicElement {
  Q_OBJECT

public:
  Project(Workspace * workspace=0, const QString name="project");
  ~Project();
  virtual const QString tagName() const { return "Project"; }

public:
  enum GraphType {
    GRAPH,
    NEURAL,
    SAMPLE,
    SPLIT,
    POLYNOMIAL,
    PROCESS
  };

  bool load();
  bool save();
  void init()                                { xml_read(); }
  void setDoubleBuffering(bool yes=true)     { if(canvas) canvas->setDoubleBuffering(yes); }
  void enlarge()                             { if(canvas) canvas->resize(canvas->width()*4/3, canvas->height()*4/3); }
  void shrink()                              { if(canvas) canvas->resize(canvas->width()*3/4, canvas->height()*3/4); }
  void updateCanvas()                        { if(canvas) canvas->update(); }
  void setAxesVisible(bool yes=true)         { if(canvas) canvas->setAxesVisible(yes); }
  void setAnimated(bool yes)                 { if(canvas) canvas->setAnimated(yes); }
  Collection * getGraphs()                   { return & graphs; }
  ScientificCanvas * getCanvas()             { return canvas; }
  Workspace * getWorkspace()                 { return workspace; }
  QString getPlotFileName();
 
  QString makeName(QString suggestion, const bool alwaysNumber=true);
  static QString makeName(QString suggestion, const bool alwaysNumber, vector<BasicElement * > * myGraphs);


  void setFocus()                                     { if(canvas) canvas->setFocus(); }
  void nameChanged(const int ID)                      { emit fillListBox(); }
  void addGraph(GraphType type, bool show=true);
  void addGraph(BasicElement * graph);
  void die()                                          { emit die(this); }
  virtual bool xml_read(QDomElement * element=0);
  virtual void xml_list();
  virtual void refresh();
  virtual void mousePressed(const QMouseEvent * event)
    { if(selectedElement) selectedElement->mousePressed(event); }
  virtual void mouseDoubleClicked(const QMouseEvent * event)
    { if(selectedElement) selectedElement->mouseDoubleClicked(event); }
  virtual void mouseMoved(const QMouseEvent * event)
    { if(selectedElement) selectedElement->mouseMoved(event); }
  virtual void mouseReleased(const QMouseEvent * event)
    { if(selectedElement) selectedElement->mouseReleased(event); }

  virtual void setValid(const bool yes=true);
  void refreshElement(const int ID);
  void deleteGraph(const int ID);
  int getSelectedElement() const                   { return  selectedElementID; }
  void selectElement(const int ID=0);
  void highlightElement(const int ID=0);

signals:
  void valueChanged();
  void elementSelected(int ID);
//  void elementHighlighted(int ID);
  void die(Project * project);
  void projectNameChanged();
  void fillListBox();

protected:
  void destroy();
  QString getDirName(QString fileName);
  
private:
  ScientificCanvas * canvas;
  int selectedElementID;
  int highlightedElementID;
  BasicElement * selectedElement;
  BasicElement * highlightedElement;
  QWidget * parent;
  Workspace * workspace;
  Collection graphs;
  QString fileName;
  QString dirName;
  QString plotDirName;
  bool refreshing;
};

#endif
