/***************************************************************************
                          ruler.h  -  description
                             -------------------
    begin                : Wed Mar 6 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RULER_H
#define RULER_H

#include "macros.h"
#include "basicelement.h"
#include "scale.h"

#include <qcanvas.h>

class QDomElement;
class ScientificCanvas;
class RulerLine;
class RulerText;

//-----------------------------------------------------class RulerLine

class RulerLine : public QCanvasLine, public BasicItem {
public:
  RulerLine(const int ID, ScientificCanvas * canvas);
};

//-----------------------------------------------------class RulerText

class RulerText : public QCanvasText, public BasicItem {
public:
  RulerText(const int ID, ScientificCanvas * canvas);
};

//-----------------------------------------------------class Ruler

class Ruler : public BasicElement {
  Q_OBJECT

public:
  Ruler(const int ID, ScientificCanvas * canvas, const QString name="ruler",
        QDomElement * element=0);
  ~Ruler();
  virtual const QString tagName() const { return "Ruler"; }
  virtual void setValid(const bool yes);
  void setMargin(const int & newMargin = MARGIN_DEFAULT ){ margin = newMargin;}
  const int & getMargin() const                         { return margin; }
  double setAngle(const double & angle=0);
  double getAngle() const;
  void setInvTangens(const double & newInvTangens=1);
  const double & getTangens() const                     { return tangens; }
  const double & getInvTangens() const                  { return invTangens; }
  void setPoints(const int & x1,const int & y1);
  void setPoints(const QPoint & point)                  { setPoints(point.x(),point.y()); }
  void setPoints(int x1,int y1,int x2,int y2 );
  void setVisible(bool yes=true);
  void setNumbers(bool yes=true)                         { numbers = yes; }
  void show()                                            { setVisible(true); }
  void hide()                                            { setVisible(false); }
  void setOrigin(const int & x=0, const int & y=0);
  Scale * getScale()                                     { return scale; }
  void setCanvasSize( const int newWidth, const int newHeight )
    { width = newWidth, height = newHeight; }

public: // overridden
  virtual void refresh();
  virtual void toggleColor();
  virtual void setPen(const QPen & pen);
  virtual bool xml_read(QDomElement * element);
  virtual void xml_list();
  virtual void setID(const int newID);

public slots:
  void setTangens(double newTangens);

signals:
  void invTangensChanged(double newTangens);

private:
  void destroy();
  void makeRule();
  int virtualWidth() const                      { return (horizontal ? width : height)-margin;}
  int virtualHeight() const                     { return (horizontal ? height : width)-margin;}
  bool inWidth(const int & point) const         { return (point >= margin && point <= width-margin);}
  bool inHeight(const int & point) const        { return (point >= margin && point <= height-margin);}
  bool inVirtualWidth(const int & point) const  { return (point >= margin && point <= virtualWidth());}
  bool inVirtualHeight(const int & point) const { return (point >= margin && point <= virtualHeight());}
  bool setHorizontal(const bool & yes=true);
  void destroyLineList();
  void destroyTextList();
  void makeSpikes();
  double smooth(const double & value);

private: // Private attributes
  ScientificCanvas * canvas;
  Scale * scale;
  RulerLine mainLine;
  bool horizontal;
  bool visible;
  bool numbers;
  int margin;
  double spikeUnit;
  double tangens, invTangens;
  double * virtualTangens, * virtualInvTangens;
  int x1,x2,y1,y2;
  int *vx1, *vx2, *vy1, *vy2;
  int * virtualOriginX, * virtualOriginY;
  int originX, originY;
  double zero,length;
  int width, height;

  RulerLine ** lineList;
  int lineListSize;

  RulerText ** textList;
  int textListSize;

  static double deg2rad;
};

#endif
