/*************f**************************************************************
                          sample.cpp  -  description
                             -------------------
    begin                : Wed May 8 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "data.h"
#include "sample.h"
#include "project.h"
#include "scientificcanvas.h"
#include "axes.h"
#include "varreader.h"
#include "process.h"
#include "polynomial.h"
#include "enhancedprogressbar.h"
#include "distribution.h"

#include <qdom.h>

#include <cmath>
#include <ctime>

using namespace std;

Sample::SourceEnumeration Sample::sourceEnumeration;
Sample::GrainEnumeration Sample::grainEnumeration;
Sample::DistributionEnumeration Sample::distributionEnumeration;

Sample::SourceEnumeration::SourceEnumeration()
  : DomEnumeration("source enumeration")
{
  insert("draw", DRAW);
  insert("file", FILE);
  insert("merge", MERGE);
  insert("polynomial", POLYNOMIAL);
  insert("process", PROCESS);
}

Sample::GrainEnumeration::GrainEnumeration()
  : DomEnumeration("grain enumeration")
{
  insert("uniform", 0);
  insert("9", 9);
  insert("8", 8);
  insert("7", 7);
  insert("6", 6);
  insert("5", 5);
  insert("4", 4);
  insert("3", 3);
  insert("2", 2);
  insert("1", 1);
}

Sample::DistributionEnumeration::DistributionEnumeration()
  : DomEnumeration("distribution enumeration")
{
  insert("---", ZERO);
  insert("Gaussian", GAUSSIAN);
  insert("Cauchy", CAUCHY);
  insert("Exponential", EXPONENTIAL);
  insert("uniform", UNIFORM);
}

Sample::Sample(Project * project, const QString name, QDomElement * element)
  : Graph(project, name),
    vanDerMonde(getID(),  & points),
    distribution(0)
{
  init();
  xml_read(element);
  refresh();
}
Sample::Sample(Sample * other)
  : Graph(other->project),
    vanDerMonde(getID(), & points),
    distribution(0)
{
  init();
  xml_copy(other);
  setElementName(project->makeName(getElementName()));
  refresh();
}
void Sample::init(){
  points.setDefaultColor("navy");
//  points.setPreserve(true);
  canCopy = true;
  elementEnumeration = new DomEnumeration("element enumeration");
}
Sample::~Sample(){
  if(elementEnumeration) delete elementEnumeration;
  elementEnumeration = 0;
  if( distribution ) delete distribution;
  distribution = 0;
  destroy();
}
void Sample::destroy(){
  Graph::destroy();
  vanDerMonde.destroy();
  mean = 0;
  sigma = 0;
  sum = 0;
  sumOfSquares = 0;
}
void Sample::setID(const int ID){
  Graph::setID(ID);
  vanDerMonde.setID(ID);
}
bool Sample::xml_read(QDomElement * element){
  destroy();
  Graph::xml_read(element);
  vanDerMonde.xml_read(element);
  if(element){
    QDomElement myElement = element->elementsByTagName("Distribution").item(0).toElement();
    makeDistribution( & myElement);
  }
  valid = false;
  makeControl();
  setID( getID() );

  return true;
}
void Sample::makeControl(){
  controlSource = source;
  controlTake = take;
  controlGrains = grains;
  controlDistro = distro;
  makeControlID();
}
void Sample::makeControlID(){
  controlProcessID = processID;
  controlPolynomialID = polynomialID;
  controlSample_1_ID = sample_1_ID;
  controlSample_2_ID = sample_2_ID;
  controlSample_3_ID = sample_3_ID;
  controlSample_4_ID = sample_4_ID;
}
void Sample::xml_list(){
  switch(source){
    case (int)PROCESS :
      elementEnumeration->init(getParentCollection(), "Process");
      break;
    case (int)POLYNOMIAL :
      elementEnumeration->init(getParentCollection(), "Polynomial");
      break;
    case (int)MERGE :
      elementEnumeration->init(getParentCollection(), "Sample");
      elementEnumeration->removeID(getID());
      break;
  }
  Graph::xml_list();
  xml_bool("valid", valid, true, HIDE);
  vanDerMonde.setID(getID());
  xml_object( & vanDerMonde );
  xml_double("standard deviation", sigma, 0);
  xml_double("mean", mean, 0);
  xml_int("size", pointCount, 0,
    ( source == (int)PROCESS || source == (int)POLYNOMIAL ? EDIT : SHOW ) );
  xml_enum( & grainEnumeration, "grains", grains, 0,
    ( source == (int)PROCESS || source == (int)POLYNOMIAL ? SHOW : HIDE ) );
  bool noiseFactor =  ( source == (int)PROCESS || source == (int)POLYNOMIAL );
  xml_int("seed", seed, 0, 
    ( source == (int)PROCESS || source == (int)POLYNOMIAL ? EDIT : HIDE ) );
  xml_enum( & distributionEnumeration, "distribution", distro, (int)ZERO,
    ( noiseFactor ? SHOW : HIDE ), distribution);
  xml_double("sum of squares", sumOfSquares, 0, HIDE);
  xml_double("sum", sum, 0, HIDE);
  xml_double("end", polynomialEnd, 10,
    ( source == (int)POLYNOMIAL ? EDIT : HIDE ) );
  xml_double("begin", polynomialBegin, 0,
    ( source == (int)POLYNOMIAL ? EDIT : HIDE ) );
  xml_select(elementEnumeration, "polynomial", polynomialID, -1,
    ( source == (int)POLYNOMIAL ? SHOW : HIDE ) );
  xml_select(elementEnumeration, "process", processID, -1,
    ( source == (int)PROCESS ? SHOW : HIDE ) );
  xml_select(elementEnumeration, "sample D", sample_4_ID, -1,
    ( source == (int)MERGE ? SHOW : HIDE ) );
  xml_select(elementEnumeration, "sample C", sample_3_ID, -1,
    ( source == (int)MERGE ? SHOW : HIDE ) );
  xml_select(elementEnumeration, "sample B", sample_2_ID, -1,
    ( source == (int)MERGE ? SHOW : HIDE ) );
  xml_select(elementEnumeration, "sample A", sample_1_ID, -1,
    ( source == (int)MERGE ? SHOW : HIDE ) );
  xml_enum( & sourceEnumeration, "source", source, (int)DRAW);
  xml_bool("take", take, false, ( source == (int)DRAW ? HIDE : EDIT ) );
  xml_object(distribution, HIDE);
}
void Sample::recalculate(){
  valid = true;
  points.recalculate();
  updateCanvas();
}
void Sample::refresh(){
  vanDerMonde.refresh();
  if(controlSource != source){
    controlSource = source;
    if( (source == (int)POLYNOMIAL || source == (int)PROCESS ) && pointCount < 1)
      pointCount = 300;
    xml_edit();
  }
  if(controlGrains != grains){
    controlGrains = grains;
    take = false;
    emit valueChanged();
  }
  if(controlDistro != distro){
    controlDistro = distro;
    makeDistribution();
    xml_edit();
    take = false;
  }
  if(    controlPolynomialID != polynomialID || controlProcessID != processID
      || controlSample_1_ID != sample_1_ID || controlSample_2_ID != sample_2_ID
      || controlSample_3_ID != sample_3_ID ){
    makeControlID();
    take = false;
    emit valueChanged();
  }
  if(controlTake != take){
    if(      ( source == (int)PROCESS && processID < 0 )
          || ( source == (int)POLYNOMIAL && polynomialID < 0) ) take = false;
    controlTake = take;
    if(take){
      switch(source){
        case (int)FILE :
          take = loadSample();
          break;
        case (int)MERGE :
          take = mergeSamples();
          break;
        case (int)PROCESS : 
        case (int)POLYNOMIAL :
          take = takeSample();
          break;
      }
      fitCanvas();
      recalculate();
      statistics();
    } else {
      destroy();
    }
    updateCanvas();
    emit valueChanged();
  }
  Graph::refresh();
}
bool Sample::loadSample(){
  destroy();
  VarReader reader;
  if( ! reader.load(&points) ) return false;
  return true;
}
bool Sample::takeSample(){
  destroy();
  Collection * parentCollection = getParentCollection();
  if( ! parentCollection || ! pointCount ) return false;
  Process * process = 0;
  Polynomial * polynomial = 0;
  double begin, end, xVal;
  if( source == (int)PROCESS ){
    process = dynamic_cast<Process *>(parentCollection->IDToElement(processID));
    if( ! process ) return false;
    begin = process->getBegin();
    end   = process->getEnd();
  } else if( source == (int)POLYNOMIAL ){
    polynomial = dynamic_cast<Polynomial *>(parentCollection->IDToElement(polynomialID));
    if( ! polynomial ) return false;
    begin = polynomialBegin;
    end = polynomialEnd;
  } else return false;
  double range( end - begin );
  double rangeDivRandMax = (double)range / RAND_MAX;
  double grainsDivRandMax = (double)grains / RAND_MAX;
  EnhancedProgressBar progress( "taking points ...", pointCount);
  srand( seed ? seed : time(0) );
  if( grains ){
    double grain[grains];
    for(int i = 0; i<grains; i++){
      grain[i] = ( ( i + 0.5 ) * range / grains ) + begin ;
    }
    double xSigma = range / ( 3 * ( grains + 2 ) );
    for(int i = 0; i<pointCount;){
      if ( progress.check(63, i) ) break;
      int myGrain = (int)floor( grainsDivRandMax * (double)rand() );
      if(myGrain >= grains) continue;
      do {
        xVal = gaussianNoise(xSigma) + grain[myGrain];
      } while (xVal < begin || xVal > end);
      points.push_back(   xVal, getNoise() +
                        ( source == (int)PROCESS ? process->getY(xVal)
                                                 : polynomial->getY(xVal) ) );
      i++;
    }
  } else {
    for(int i = 0; i<pointCount; i++){
      if ( progress.check(63, i) ) break;
      xVal = begin + rangeDivRandMax * (double)rand();
      points.push_back(   xVal, getNoise() +
                        ( source == (int)PROCESS ? process->getY(xVal)
                                                 : polynomial->getY(xVal) ) );
    }
  }
  return true;
}
double Sample::getNoise(){
  return distribution ? distribution->draw() : 0;
}
void Sample::fitCanvas(){
  double up, down, left, right;
  points.bounds(up, down, left, right);
  project->getCanvas()->getAxes()->expand(up, down, left, right);
}
void Sample::mousePressed(const QMouseEvent * event){
  if(    points.getVisible() && source == (int)DRAW
      && (event->button() & QMouseEvent::RightButton))
    addPoint(event->pos());
}
void Sample::mouseMoved(const QMouseEvent * event){
  if(    points.getVisible() && source == (int)DRAW
      && (event->state() & QMouseEvent::RightButton))
   addPoint(event->pos());
}
void Sample::addPoint(const QPoint & point){
  vanDerMonde.makeZero();
  double x, y;
  if( ! project->getCanvas()->getAxes()->posToVal(point.x(), point.y(), x, y) ){
    return;
  }
  points.push_back(point.x(), point.y(), x, y);
  pointCount = points.size();

  sum += y;
  sumOfSquares += y*y;
  mean = sum / pointCount;
  sigma = sqrt(sumOfSquares / pointCount - mean * mean);

  emit valueChanged();
  updateCanvas();
}
void Sample::makeDistribution(QDomElement * element){
  if( distribution ) delete distribution;
  distribution = 0;
  switch(distro){
    case (int)UNIFORM :
      distribution = new UniformDistribution( getID(), element);
      break;
    case (int)GAUSSIAN :
      distribution = new GaussianDistribution( getID(), element);
      break;
    case (int)EXPONENTIAL :
      distribution = new ExponentialDistribution( getID(), element);
      break;
    case (int)CAUCHY :
      distribution = new CauchyDistribution( getID(), element);
      break;
  }
}
bool Sample::mergeSamples(){
  cerr << " merge 1\n";
  Collection * parentCollection = getParentCollection();
  if( ! parentCollection ) return false;
  Sample * sample;
  sample = dynamic_cast<Sample *>(parentCollection->IDToElement(sample_1_ID));
  if( sample ) points.copy( * sample->getPoints());// else cerr << " no sample A\n";
  sample = dynamic_cast<Sample *>(parentCollection->IDToElement(sample_2_ID));
  if( sample ) points.copy( * sample->getPoints());// else cerr << " no sample B\n";;
  sample = dynamic_cast<Sample *>(parentCollection->IDToElement(sample_3_ID));
  if( sample ) points.copy( * sample->getPoints());// else cerr << " no sample C\n";
  sample = dynamic_cast<Sample *>(parentCollection->IDToElement(sample_4_ID));
  if( sample ) points.copy( * sample->getPoints());// else cerr << " no sample D\n";
  cerr << " merge 10\n";
  return points.size();
}
