/***************************************************************************
                          sample.h  -  description
                             -------------------
    begin                : Wed May 8 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SAMPLE_H
#define SAMPLE_H

#include "graph.h"
#include "vandermonde.h"
#include "domenumeration.h"

#include <vector>

using namespace std;

class Project;
class SinglePoint;
class Distribution;

class Sample : public Graph  {
public:
  Sample(Project * project, const QString name="sample", QDomElement * element=0);
  Sample(Sample * other);
  ~Sample();
  virtual const QString tagName() const { return "Sample"; }

public:
  virtual bool xml_read(QDomElement * element);
  virtual void xml_list();
  virtual void setID(const int ID);
  virtual void refresh();
  virtual void recalculate();
  virtual void mousePressed(const QMouseEvent * event);
  virtual void mouseMoved(const QMouseEvent * event);
  void init();
  void addPoint(const QPoint & point);
  int getPointCount()                             { return pointCount; }
  Graph * copyGraph()                             { return new Sample(this); }
  VanDerMonde * getVanDerMonde()                  { return & vanDerMonde; }

private:
  bool loadSample();
  bool takeSample();
  bool mergeSamples();
  double getNoise();
  void makeControl();
  void makeControlID();
  void destroy();
  void fitCanvas();
  void makeDistribution(QDomElement * element=0);

private:
  enum Source {
    NONE,
    DRAW,
    FILE,
    PROCESS,
    POLYNOMIAL,
    MERGE
  };
  
  enum SampleDistribution {
    ZERO,
    GAUSSIAN,
    CAUCHY,
    EXPONENTIAL,
    UNIFORM,
  };

  
private:
  class SourceEnumeration : public DomEnumeration{
    public: SourceEnumeration();
  };
  class GrainEnumeration : public DomEnumeration{
    public: GrainEnumeration();
  };
  class DistributionEnumeration : public DomEnumeration{
    public: DistributionEnumeration();
  };
  static SourceEnumeration sourceEnumeration;
  static GrainEnumeration grainEnumeration;
  static DistributionEnumeration distributionEnumeration;

private:
  int source, controlSource;
  int distro, controlDistro;
  bool take, controlTake;
  int grains, controlGrains;
  int seed;
  VanDerMonde vanDerMonde;
  Distribution * distribution;
  double polynomialBegin, polynomialEnd;
  int processID, controlProcessID,
      polynomialID, controlPolynomialID,
      sample_1_ID, controlSample_1_ID,
      sample_2_ID, controlSample_2_ID,
      sample_3_ID, controlSample_3_ID,
      sample_4_ID, controlSample_4_ID;
  DomEnumeration * elementEnumeration;
};

#endif
