/***************************************************************************
                          scale.cpp  -  description
                             -------------------
    begin                : Fri Apr 26 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "scale.h"

#include <qdom.h>

#include <cmath>

using namespace std;

Scale::ScaleEnumeration Scale::scaleEnumeration;
  
Scale::ScaleEnumeration::ScaleEnumeration()
  : DomEnumeration("scale enumeration")
{
  insert("simple", SIMPLE);
  insert("logarithmic", LOG);
  insert("log(log())", LOGLOG);
  insert("exponential", EXP);
  insert("sigmoid", SIGMOID);
  insert("square", SQUARE);
  insert("square root", SQUAREROOT);
}


Scale::Scale(BasicElement * parent, const QString name, QDomElement * /*element*/)
  : BasicElement(parent ? parent->getID() : 0, name),
    function(simpleScale),
    inverse(simpleInvScale),
    parent(parent)
{
  hideName();
  xml_read();
}
Scale::~Scale(){
}
bool Scale::xml_read(QDomElement * element){
  BasicElement::xml_read(element);
  controlMethod = method;
  controlStretch = stretch;
  setParameters();
  return true;
}
void Scale::xml_list(){
  BasicElement::xml_list();
  xml_bool("valid", valid, true, HIDE);
  xml_enum( & scaleEnumeration, "scale", method, SIMPLE);
  xml_double("stretchFactor", stretch, 1, EDIT);
}
void Scale::setMethod(Method newMethod){
  method = newMethod;
  refresh();
}
void Scale::refresh(){
//  cerr << getElementName() << "::refresh\n";
  if(stretch != controlStretch){
    controlStretch = stretch;
    valid = false;
  }
  if(controlMethod != method){
    controlMethod = method;
    valid = false;
    setParameters();
  }
  if( ! valid ){
    valid = true;
    parent->setValid(false);
    parent->refresh();
  }
}
void Scale::setParameters(){
  switch(method){
    case LOG :
      function = expInvScale;
      inverse = expScale;
      lowerBoundary = 1e-308;
      upperBoundary = 1e308;
      break;
    case LOGLOG :
        function = expExpInvScale;
        inverse = expExpScale;
        lowerBoundary = 1.0000001;
        upperBoundary = 1e308;
        break;
    case EXP :
      function = expScale;
      inverse = expInvScale;
      lowerBoundary = -1e308;
      upperBoundary = 709;
      break;
    case SIGMOID :
      function = sigmoidScale;
      inverse = sigmoidInvScale;
      lowerBoundary = -709;
      upperBoundary = 1e308;
      break;
    case SQUARE :
      function = squareScale;
      inverse = squareInvScale;
      lowerBoundary = -1e154;
      upperBoundary = 1e154;
      break;
    case SQUAREROOT :
      function = squareInvScale;
      inverse = squareScale;
      lowerBoundary = -1e308;
      upperBoundary = 1e308;
      break;
    case SIMPLE :
      function = simpleScale;
      inverse = simpleInvScale;
      lowerBoundary = -1e308;
      upperBoundary = 1e308;
      break;
  }
}
double Scale::simpleScale(const double & value){
  return value;
}
double Scale::simpleInvScale(const double & value){
  return value;
}
double Scale::expScale(const double & value){
  return exp(value);
}
double Scale::expInvScale(const double & value){
  return log(value);
}
double Scale::expExpScale(const double & value){
  return exp(exp(value));
}
double Scale::expExpInvScale(const double & value){
  return log(log(value));
}
double Scale::squareScale(const double & value){
  return (value >= 0 ? pow(value,2) : -pow(value,2) );
}
double Scale::squareInvScale(const double & value){
  return (value >= 0 ? sqrt(value) : -sqrt(-value) );
}
double Scale::sigmoidScale(const double & value){
  return 1 / ( 1+exp(-value) );
}
double Scale::sigmoidInvScale(const double & value){
  return log( value / (1-value) );
}
void Scale::setLeftX(const double & /*x*/){
//  intLeftX = x;
}
void Scale::setRightX(const double & /*x*/){
//  intRightX = x;
}
void Scale::setUpperY(const double & /*y*/){
//  intUpperY = y;
}
void Scale::setLowerY(const double & /*y*/){
//  intLowerY = y;
}

