/***************************************************************************
                          scale.h  -  description
                             -------------------
    begin                : Fri Apr 26 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SCALE_H
#define SCALE_H

#include "basicelement.h"
#include "domenumeration.h"

class QDomElement;

class Scale: public BasicElement {
public:
  enum Method{
    NONE,
    SIMPLE,
    LOG,
    LOGLOG,
    EXP,
    SQUARE,
    SQUAREROOT,
    SIGMOID
  };

  Scale(BasicElement * parent, const QString name="scale", QDomElement * element=0);
  ~Scale();
  virtual const QString tagName() const { return "Scale"; }

//  int operator()(const double & value)      { return (*fun)(value); }
//  double operator()(const int & position)   { return  (*inv)(position); }
  double fun(const double & value)            { return (*function)(stretch*value); }
  double inv(const double & value)            { return ((*inverse)(value))/stretch; }
  void setLeftX(const double & x);
  void setRightX(const double & x);
  void setUpperY(const double & y);
  void setLowerY(const double & y);
  double lowerBound() const                  { return lowerBoundary/stretch; }
  double upperBound() const                  { return upperBoundary/stretch; }
  void setMethod(Method method);

  virtual bool xml_read(QDomElement * element=0);
  virtual void xml_list();
  virtual void refresh();

private:
  void setParameters();
  double ( * function) (const double & value);
  double ( * inverse) (const double & value);

  static double simpleScale(const double & value);
  static double simpleInvScale(const double & value);
  static double expScale(const double & value);
  static double expInvScale(const double & value);
  static double expExpScale(const double & value);
  static double expExpInvScale(const double & value);
  static double squareScale(const double & value);
  static double squareInvScale(const double & value);
  static double sigmoidScale(const double & value);
  static double sigmoidInvScale(const double & value);

private:
  class ScaleEnumeration : public DomEnumeration {
    public: ScaleEnumeration();
  };
  static ScaleEnumeration scaleEnumeration;

private:
  BasicElement * parent;
  int intLeftX, intRightX, intLowerY, intUpperY;
  double xScale, yScale;
  int method, controlMethod;
  double lowerBoundary, upperBoundary;
  double stretch, controlStretch;
};

#endif
