/***************************************************************************
                          singlepoint.cpp  -  description
                             -------------------
    begin                : ?' ??? 1 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "singlepoint.h"
#include "axes.h"

#include <qdom.h>

SinglePoint::SinglePoint(const int ID, ScientificCanvas * canvas,
                         QDomElement * element)
  : BasicItem(ID),
    QCanvasLine(canvas),
    valid(true),
    thickness(0),
    posX(0),
    posY(0),
    valueX(0),
    valueY(0)
{
  if(element) xml_read(element);
}
SinglePoint::SinglePoint(const int ID, ScientificCanvas * canvas,
                         const int posX, const int posY, const int & thickness,
                         const bool & valid)
  : BasicItem(ID),
    QCanvasLine(canvas),
    valid(valid),
    thickness(thickness),
    posX(posX),
    posY(posY),
    valueX(0),
    valueY(0)
{
  setPoints(posX, posY, posX + thickness, posY);
}
SinglePoint::SinglePoint(const int ID, ScientificCanvas * canvas,
                         const double & valueX, const double & valueY,
                         const int & thickness, const bool & valid)
  : BasicItem(ID),
    QCanvasLine(canvas),
    valid(valid),
    thickness(thickness),
    posX(0),
    posY(0),
    valueX(valueX),
    valueY(valueY)
{}
SinglePoint::SinglePoint(const int ID, ScientificCanvas * canvas,
                         const int posX, const int posY,
                         const double & valueX, const double & valueY,
                         const int & thickness, const bool & valid)
  : BasicItem(ID),
    QCanvasLine(canvas),
    valid(valid),
    thickness(thickness),
    posX(posX),
    posY(posY),
    valueX(valueX),
    valueY(valueY)
{
  setPoints(posX, posY, posX + thickness, posY);
//  cerr << " posX " << posX << " \t posY " << posY << " \t valY " << valueY << endl;
}
SinglePoint::SinglePoint(const int ID, ScientificCanvas * canvas,
                         SinglePoint * otherPoint,
                         const int & thickness, const bool & /*valid*/)
  : BasicItem(ID),
    QCanvasLine(canvas),
    valid(true),
    thickness(thickness),
    posX(otherPoint->posX),
    posY(otherPoint->posY),
    valueX(otherPoint->valueX),
    valueY(otherPoint->valueY)
{
  setPoints(posX, posY, posX + thickness, posY);
}
bool SinglePoint::xml_read(QDomElement * element){
  valueX = element->attribute("valX", "0").toDouble();
  valueY = element->attribute("valY", "0").toDouble();
  posX = element->attribute("posX","0").toInt();
  posY = element->attribute("posY","0").toInt();
  return true;
}
bool SinglePoint::xml_write(QDomElement * parent){
  QDomElement element = parent->ownerDocument().createElement(QString("Point"));
  element.setAttribute(QString("valX"), valueX);
  element.setAttribute(QString("valY"), valueY);
  element.setAttribute("posX", posX);
  element.setAttribute("posY", posY);
  parent->appendChild(element);
  return true;
}
bool SinglePoint::recalculate(Axes * axes){
  bool val = axes->valToPos( valueX, valueY, posX, posY);
  setValid(val);
  return val;
}
