/***************************************************************************
                          singlepoint.h  -  description
                             -------------------
    begin                : ?' ??? 1 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SINGLEPOINT_H
#define SINGLEPOINT_H

#include "basicelement.h"
#include "scientificcanvas.h"

class QDomElement;
class Axes;

class SinglePoint : public BasicItem, public QCanvasLine {
public:
  SinglePoint(const int ID=0, ScientificCanvas * canvas=0, QDomElement * element = 0);
  SinglePoint(const int ID, ScientificCanvas * canvas, const int posX,
              const int posY, const double & valueX, const double & valueY,
              const int & thickness, const bool & valid=true);
  SinglePoint(const int ID, ScientificCanvas * canvas,
              const double & valueX, const double & valueY,
              const int & thickness, const bool & valid=true);
  SinglePoint(const int ID, ScientificCanvas * canvas,
              const int posX, const int posY,
              const int & thickness, const bool & valid=true);
  SinglePoint(const int ID, ScientificCanvas * canvas, SinglePoint * otherPoint,
              const int & thickness, const bool & valid=true);
  bool xml_read(QDomElement * element);
  bool xml_write(QDomElement * parent);

  int getPositionX() const                            { return posX;}
  int getPositionY() const                            { return posY;}
  QPoint getPosition() const                          { return QPoint(posX, posY); }
  void setPosition(const int & x, const int & y)      { posX = x, posY = y; }

  double getValueX() const                            { return valueX;}
  double getValueY() const                            { return valueY;}
  void setValue(const double & x, const double & y)   { valueX = x, valueY = y; }

  void setValid(const bool & yes=true)         { valid=yes; }
  bool getValid()                              { return valid; }
  bool recalculate(Axes * axes);
  void setThickness(const int newThickness)    { thickness = newThickness; }
  void makePoint()                             { setPoints(posX, posY, posX+thickness, posY); }
  void makeLine(int & oldX, int & oldY)        { setPoints(oldX, oldY, posX, posY); oldX = posX; oldY = posY; }

private:
//  void setCanvasLineVisible()                  { QCanvasLine::setVisible(visible); }

private:
  bool valid;
  int thickness;
  int posX, posY;
  double valueX, valueY;
};

#endif
