/***************************************************************************
                          split.cpp  -  description
                             -------------------
    begin                : ?' ??? 6 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "split.h"
#include "project.h"
#include "sample.h"

//#include <algo.h>
#include <algorithm>

using namespace std;

QString Split::colorA = "red";
QString Split::colorB = "black";

Split::MethodEnumeration Split::methodEnumeration;
  
Split::MethodEnumeration::MethodEnumeration()
  : DomEnumeration("split method enumeration")
{
  insert("percentage", Split::PERCENT);
  insert("number", Split::NUMBER);
}

Split::Split(Project * project, const QString name, QDomElement * element)
  : Graph(project, name),
    sampleEnumeration(0),
    splitEnumeration(0),
    sample_1_ID(-1),
    sample_2_ID(-1)
{
  init();
  xml_read(element);
  if(sample_1_ID < 0) sample_1_ID = makeSample(getElementName() + " A ", colorA);
  if(sample_2_ID < 0) sample_2_ID = makeSample(getElementName() + " B ", colorB);
  refresh();
}
Split::~Split(){
  if(sampleEnumeration) delete sampleEnumeration;
  sampleEnumeration = 0;
  if(splitEnumeration) delete splitEnumeration;
  splitEnumeration = 0;
}
void Split::init(){
  points.setDefaultColor("navy");
  sampleEnumeration = new DomEnumeration("sample enumeration");
  splitEnumeration = new DomEnumeration("split enumeration");
}
int Split::makeSample(QString name, QString color){
  Sample * sample = new Sample(project, name);
  sample->setDefaultColor(color);
  project->addGraph(sample);
  return sample->getID();
}
void Split::makeControl(){
  controlMethod = method;
  controlSplitSampleID = splitSampleID;
  controlPercent = percent;
  controlNumber = number;
  controlSplit = split;
}
void Split::destroy(){
  Graph::destroy();
}
bool Split::xml_read(QDomElement * element){
  destroy();
  bool result = Graph::xml_read(element);
  valid = false;
  makeControl();
  return result;
}
void Split::xml_list(){ // does not call Graph::xml_list() !!
  BasicElement::xml_list();
  sampleEnumeration->init(getParentCollection(), "Sample");
  sampleEnumeration->add(getParentCollection(), "Merge");
  sampleEnumeration->removeID(sample_1_ID);
  sampleEnumeration->removeID(sample_2_ID);
  splitEnumeration->init(getParentCollection(), "Sample");
  xml_bool("copy", copyBool, false, (canCopy ? EDIT : HIDE));
  xml_bool("delete", deleteBool, false, EDIT);
  xml_select(splitEnumeration, "sample B", sample_2_ID, sample_2_ID);
  xml_select(splitEnumeration, "sample A", sample_1_ID, sample_1_ID);
  xml_bool("valid", valid, true, HIDE);
  xml_int("number", number, 100, ( method == (int)NUMBER ? EDIT : HIDE ) );
  xml_double("percentage", percent, 20, ( method == (int)PERCENT ? EDIT : HIDE ) );
  xml_enum( & methodEnumeration, "method", method, (int)PERCENT);
  xml_select(sampleEnumeration, "sample", splitSampleID, -1);
  xml_bool("split", split, false, EDIT);
}
void Split::refresh(){
  Graph::refresh();
  if(controlPercent != percent){
    if(percent >= 0 && percent <= 100){
      controlPercent = percent;
    } else {
      percent = controlPercent;
    }
    split = false;
    emit valueChanged();
  }
  if(controlNumber != number){
    controlNumber = number;
    split = false;
    emit valueChanged();
  }
  if(controlSplit != split){
    controlSplit = split;
    if(split) splitSample();
    emit valueChanged();
  }
  if(controlMethod != method){
    controlMethod = method;
    xml_edit();
  }
  if( controlSplitSampleID != splitSampleID ){
    controlSplitSampleID = splitSampleID;
    setElementName(project->makeName("split " + sampleEnumeration->stringAtValue(splitSampleID), false));
    xml_edit();
  }
}
bool Split::splitSample(){
  Points * parent, * points_1, * points_2;
  Sample * sample_1, * sample_2;
  int newID;
  Collection * parentCollection = getParentCollection();
  if( ! parentCollection ) return false;
  Sample * sample_0 = dynamic_cast<Sample *>(parentCollection->IDToElement(splitSampleID));
  if( ! sample_0 ) return false;
  parent = sample_0->getPoints();
  if( parent->empty() ) return false;
  sample_1 = dynamic_cast<Sample *>(parentCollection->IDToElement(sample_1_ID));
  if( sample_1 ) points_1 = sample_1->getPoints();
  else {
    sample_1_ID = makeSample(getElementName() + " A", colorA);
    sample_1 = dynamic_cast<Sample *>(parentCollection->IDToElement(sample_1_ID));
    if( ! sample_1 ) return false;
  };
  sample_2 = dynamic_cast<Sample *>(parentCollection->IDToElement(sample_2_ID));
  if( sample_2 ) points_2 = sample_2->getPoints();
  else {
    sample_2_ID = makeSample(getElementName() + " B", colorB);
    sample_2 = dynamic_cast<Sample *>(parentCollection->IDToElement(sample_2_ID));
    if( ! sample_2 ) return false;
  }
  int myNumber = (method == (int)NUMBER ? number : (int)percent * parent->size() / 100);
  if (myNumber > (int)parent->size()) myNumber = parent->size();
  if (myNumber < 0) myNumber = 0;
  if (myNumber > (int)parent->size()/2) {
    points_1 = sample_1->getPoints();
    points_2 = sample_2->getPoints();
    newID = sample_2->getID();
  } else {
    points_2 = sample_1->getPoints();
    points_1 = sample_2->getPoints();
    newID = sample_1->getID();
  }
  points_1->clear();
  points_2->clear();

  if ( ! points_1->copy( * parent) ) return false;
  if( points_1->size() > 1)
    random_shuffle( points_1->begin(),
                    points_1->begin() + (points_1->size() - 1) );
  for(int i=0; i<myNumber; i++){
    SinglePoint * point = points_1->back();
    point->setID(newID);
    points_1->pop_back();
    points_2->push_back(point);
  }
  sample_1->setVisible();
  sample_2->setVisible();
  sample_1->statistics();
  sample_2->statistics();
  parent->setVisible(false);
  
  return true;
}
