/***************************************************************************
                          timeseries.h  -  description
                             -------------------
    begin                : Fri May 10 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TIMESERIES_H
#define TIMESERIES_H

#include "basicelement.h"

class QDomElement;

class TimeSeries: public BasicElement {
public: 
  TimeSeries(const int ID=0, const QString name="timeSeries", QDomElement * element=0);
  ~TimeSeries();
  virtual const QString tagName() const { return "TimeSeries"; }

  virtual void init(){;}          // from here on
  virtual double next(double /*x*/=0)                  { return 0; } // from here on

  virtual void xml_list();
  void setSystemNoise(const double & noise)        { system_noise = noise; }
  double systemNoise();
  void start(const int newSeed = 0);
  int getSteps()                                   { return steps; }

protected:
  double system_noise;
  int seed;
  int steps;
  double offset;
  double scale;
  double myScale;
  double density;
};

//-------------------------------------------------class Autoregression

class AutoSeries: public TimeSeries {
public:
  AutoSeries(const int ID=0, QDomElement * element = 0);

public:
  virtual void xml_list();
  virtual void init();
  virtual double next(double x=0);

private:
  double a_param, b_param, c_param, d_param, e_param, f_param;
  double y_0, y_1, y_2, y_3, y_4, y_5;
};

//-------------------------------------------------class Sinus

class SinusSeries: public TimeSeries {
public:
  SinusSeries(const int ID=0, QDomElement * element = 0);

public:
  virtual void xml_list();
  virtual void init();
  virtual double next(double x=0);

private:
  double s_1, s_2, s_3, s_4, s_5;
  double f_1, f_2, f_3, f_4, f_5;
};

//-------------------------------------------------class Henon

class HenonSeries: public TimeSeries {
public:
  HenonSeries(const int ID=0, QDomElement * element = 0);

public:
  virtual void xml_list();
  virtual void init();
  virtual double next(double x=0);

private:
  double a_param, b_param;
  double y_0, y_1, z;
};

//-------------------------------------------------class Logistic

class LogisticSeries: public TimeSeries {
public:
  LogisticSeries(const int ID=0, QDomElement * element = 0);

public:
  virtual void xml_list();
  virtual void init();
  virtual double next(double x=0);

private:
  double a_param;
  double y_0, y_1;
};

//-------------------------------------------------class Lorenz

class LorenzSeries: public TimeSeries {
public:
  LorenzSeries(const int ID=0, QDomElement * element = 0);

public:
  virtual void xml_list();
  virtual void init();
  virtual double next(double x=0);

private:
  double a_param, b_param, c_param;
  double y_0, y_1, z_0, z_1, w_0, w_1;
};

//-------------------------------------------------class Pendulum

class PendulumSeries: public TimeSeries {
public:
  PendulumSeries(const int ID=0, QDomElement * element = 0);

public:
  virtual void xml_list();
  virtual void init();
  virtual double next(double x=0);

private:
  double constant, freq, orbit;
  double y_0, y_1, z_0, z_1, t;
};

//-------------------------------------------------class Step

class StepSeries: public TimeSeries {
public:
  StepSeries(const int ID=0, QDomElement * element = 0);
  ~StepSeries();

public:
  virtual void xml_list();
  virtual void init();
  virtual double next(double x=0);

private:
  int * step;
  int stepCount;
  int stepNumber;
  double a;
};


//-------------------------------------------------class Thom

class ThomSeries: public TimeSeries {
public:
  ThomSeries(const int ID=0, QDomElement * element = 0);

public:
  virtual void xml_list();
  virtual void init();
  virtual double next(double x=0);

private:
  double a_param, b_param, c_param, d_param;
  double y_0, y_1, z_0, z_1;
};

#endif
