/***************************************************************************
                          vandermonde.h  -  description
                             -------------------
    begin                : ?' ??? 4 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef VANDERMONDE_H
#define VANDERMONDE_H

#include "basicelement.h"
#include "matrix.h"

//#define MAPM double
//#define MM_Zero 0

class Points;
class EnhancedProgressBar;

class VanDerMonde : public BasicElement {
  Q_OBJECT

public:
  VanDerMonde(Points * points);
  VanDerMonde(const int ID=0, Points * points=0, QDomElement * element=0);
  VanDerMonde(const int ID, Points * points, VanDerMonde * other);
  ~VanDerMonde();
  virtual const QString tagName() const { return "VanDerMonde"; }

  virtual bool xml_write(QDomElement * element);
  virtual bool xml_read(QDomElement * parent);
  virtual void xml_list();
  virtual void refresh();

  void init();

  VanDerMonde & operator+=(VanDerMonde & other);
  bool leastSquare(const int newOrder, EnhancedProgressBar * dialog=0);
  bool processData(EnhancedProgressBar * dialog=0);
  bool makeTriangular(EnhancedProgressBar * dialog=0);
  void makeZero();
  double determinant(const int myOrder)                      { return matrix.determinant(myOrder); }
  double logDeterminant(const int myOrder, bool & valid)     { return matrix.logDeterminant(myOrder, valid); }
  double variance(Matrix & parameter, int parameterOrder);
  bool makeParameter(Matrix & parameter, int * precisionArray, int parameterOrder);
  void setOrder(int newOrder);
  int getOrder() const                                       { return order; }
  int getSampleSize();
  void destroy();
  void setPoints(Points * newPoints)                         { points = newPoints; }

signals:
  void valueChanged();

private:
  void makeFalse();
  void makeControl();

private:
  Points * points;
  Matrix matrix, sumX, sumY;
  MAPM squareY;
  int precision;
  int order, controlOrder, order2;
  bool isProcessed, isTriangular;
  bool controlIsProcessed, controlIsTriangular;
};

#endif
