/***************************************************************************
                          varreader.cpp  -  description
                             -------------------
    begin                : ?' ??? 22 2002
    copyright            : (C) 2002 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "varreader.h"
#include "points.h"
#include "enhancedprogressbar.h"
#include "project.h"

#include <qfile.h>
#include <qfiledialog.h>
#include <qinputdialog.h>
#include <qtextstream.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qtextedit.h>
#include <qlineedit.h>
#include <qstringlist.h>
#include <qlabel.h>
#include <qwidgetstack.h>
#include <qtable.h>
#include <qregexp.h>
#include <qvalidator.h>
#include <qapplication.h>
#include <qsizepolicy.h>
#include <qmessagebox.h>

VarReader::VarReader()
  : QDialog(qApp->mainWidget(), "x-y dialog", true),
    tableFilled(false),
    points(0)
{
  myRegExp = new QRegExp(" ");
  xRegExp = new QRegExp();
  yRegExp = new QRegExp();
  radioButtons = new QRadioButton * [3];
}
VarReader::~VarReader(){
  delete myRegExp;
  myRegExp = 0;
  delete xRegExp;
  xRegExp = 0;
  delete yRegExp;
  yRegExp = 0;
  delete[] radioButtons;
  radioButtons = 0;
}
bool VarReader::load(Points * myPoints){
  points = myPoints;
  QString fileName = QFileDialog::getOpenFileName();
  if (fileName.isNull() ) return false;

  QFile file(fileName);
  if( ! file.open(IO_ReadOnly)){
    QString message("file " + fileName + " is READONLY");
    QMessageBox::warning( qApp->mainWidget(),
                          QString::null, message,
                          QMessageBox::Ok,
                          QMessageBox::NoButton);
    return false;
  }

  QTextStream stream(&file);
  while(1){
    QString line = stream.readLine();
    if(line.isNull()) break;
    text += line.simplifyWhiteSpace();
    text += "\n";
  }
  file.close();
  
  if(text.isEmpty()) return false;

  buildDialog();

  exec();
  if(result() == QDialog::Accepted ){
    readPoints();
    return true;
  } else {
    return false;
  }
}
void VarReader::buildDialog(){
  QSizePolicy sizePolicy = this->sizePolicy();
  sizePolicy.setHorData(QSizePolicy::Expanding);
  this->setSizePolicy(sizePolicy);

  setSizeGripEnabled(true);
  QVBox * mainBox = new QVBox( this );
  mainBox->setSpacing(10);
  mainBox->setMargin(10);
  mainBox->setMinimumSize(800,600);

  browserStack = new QWidgetStack(mainBox);
  sizePolicy = browserStack->sizePolicy();
  sizePolicy.setVerData(QSizePolicy::Expanding);
  browserStack->setSizePolicy(sizePolicy);

  browser = new QTextEdit(browserStack);
  browser->setTextFormat(Qt::RichText);
  browser->setReadOnly(true);

  table = new QTable(browserStack);
  table->setSelectionMode(QTable::NoSelection);
  table->setReadOnly(true);
  table->setLeftMargin(0);

  QHBox * inputBox = new QHBox(mainBox);
  inputBox->setSpacing(40);
  inputStack = new QWidgetStack(inputBox);
  inputStack->setMinimumWidth(540);
  regexBox = new QVBox(inputStack);
  regexBox->setSpacing(10);
  columnBox = new QVBox(inputStack);
  columnBox->setSpacing(10);
  editBox = new QVBox(inputStack);
  editBox->setSpacing(10);
  QVBox * radioBox = new QVBox(inputBox);
  radioBox->setSpacing(10);
  QVBox * buttonBox = new QVBox(inputBox);
  buttonBox->setSpacing(10);

  QIntValidator * validator = new QIntValidator(0, 20, this );

  QHBox * xBox = new QHBox(regexBox);
  xBox->setSpacing(10);
  new QLabel(" x: ", xBox);
  xLine = new QLineEdit("^\\D*(\\d+\\.?\\d*)", xBox);
  new QLabel("   ( ) #", xBox);
  xCap = new QLineEdit("1", xBox);
  xCap->setValidator(validator);
  xCap->setFixedWidth(50);

  QHBox * yBox = new QHBox(regexBox);
  yBox->setSpacing(10);
  new QLabel(" y: ", yBox);
  yLine = new QLineEdit("^(\\D*\\d+\\.?\\d*){1,1}\\D*(\\d+\\.?\\d*)", yBox);
  new QLabel("   ( ) #", yBox);
  yCap = new QLineEdit("2", yBox);
  yCap->setValidator(validator);
  yCap->setFixedWidth(50);

  QHBox * columnSplitter = new QHBox(columnBox);
  columnSplitter->setSpacing(10);
  new QLabel(" devide columns by regex: ", columnSplitter);
  splitLine = new QLineEdit("[ \\t]+", columnSplitter);
  QHBox * columnSelect = new QHBox(columnBox);
  columnSelect->setSpacing(20);
  columnSelect->setMaximumWidth(300);
  new QLabel(" x column: ", columnSelect);
  xColumn = new QLineEdit("1", columnSelect);
  xColumn->setFixedWidth(50);
  new QLabel(" y column: ", columnSelect);
  yColumn = new QLineEdit("2", columnSelect);
  yColumn->setFixedWidth(50);

//  Box->setSpacing(10);
  radioButtons[regex]   = new QRadioButton("regex",   radioBox);
  radioButtons[column] = new QRadioButton("columns", radioBox);
//  radioButtons[edit]    = new QRadioButton("edit",    radioBox);

  connect(radioButtons[regex],   SIGNAL(clicked()), this, SLOT(regexRadio()));
  connect(radioButtons[column], SIGNAL(clicked()), this, SLOT(columnRadio()));
//  connect(radioButtons[edit],    SIGNAL(clicked()), this, SLOT(editRadio()));

  QPushButton * apply  = new QPushButton("&Apply",  buttonBox);
  QPushButton * ok     = new QPushButton("&OK",     buttonBox);
  QPushButton * cancel = new QPushButton("&Cancel", buttonBox);

  connect(apply,  SIGNAL(clicked()), this, SLOT(apply ()));
  connect(ok,     SIGNAL(clicked()), this, SLOT(ok()));
  connect(cancel, SIGNAL(clicked()), this, SLOT(reject()));

  currentRadio = column;
  regexRadio();
}
void VarReader::ok(){
  if(currentRadio == regex || currentRadio == column){
    accept();
  }
}
void VarReader::regexRadio(){
  radioButtons[regex]->setChecked(true);
  if(currentRadio == regex) return;
  radioButtons[currentRadio]->setChecked(false);
  currentRadio = regex;
  browserStack->raiseWidget(browser);
  inputStack->raiseWidget(regexBox);
  makeRegex();
}
void VarReader::columnRadio(){
  radioButtons[column]->setChecked(true);
  if(currentRadio == column) return;
  radioButtons[currentRadio]->setChecked(false);
  currentRadio = column;
  browser->setText("");
  inputStack->raiseWidget(columnBox);
  if( ! tableFilled ) makeColumn();
  browserStack->raiseWidget(table);
}
/*void VarReader::editRadio(){
  radioButtons[edit]->setChecked(true);
  if(currentRadio == edit) return;
  radioButtons[currentRadio]->setChecked(false);
  currentRadio = edit;
  browserStack->raiseWidget(browser);
  table->setNumRows(0);
  table->setNumCols(0);
  inputStack->raiseWidget(editBox);
  makeEdit();
}*/
void VarReader::makeColumn(){
  table->setNumRows(0);
  table->setNumCols(0);
  QRegExp splitter(  splitLine->text() );
  QTextStream stream(&text, IO_ReadOnly);
  QString line;
  QStringList list;
  int columns = 0;
  int lineNum = text.contains('\n');
  EnhancedProgressBar progress( "building table ...", lineNum);
  for(int row = 0; true; row++){
    if ( progress.check(row) ){
      regexRadio();
      return;
    }
    table->insertRows(row);
    line = stream.readLine();
    if( line.isNull() ) break;
    list = QStringList::split(splitter, line);
//    cerr << " listSize: " << list.size() << " line: " << line << endl;
    if(columns < (int)list.size()){
      table->insertColumns(columns , list.size() - columns);
      columns = list.size();
    }
    int col=0;
    for(QStringList::iterator iter = list.begin(); iter != list.end(); iter++, col++){
      table->setText(row,col,*iter);
    }
  }
  tableFilled = true;
}
void VarReader::makeEdit(){};
void VarReader::makeRegex(){
  searchText = "<qt>";
  QTextStream stream(&text, IO_ReadOnly);
  QString line="";
  bool colorBool = true;
  while(1){
    if(colorBool){
      colorBool = false;
    } else {
      colorBool = true;
    }
    line = stream.readLine();
    if( line.isNull() ) break;
    xRegExp->search(line);
    yRegExp->search(line);
    int xPos = xRegExp->pos(xCapture);
    int yPos = yRegExp->pos(yCapture);
    if(xPos < 0) xPos = 0;
    if(yPos < 0) yPos = 0;
    QString xMatch = xRegExp->cap(xCapture);
    QString yMatch = yRegExp->cap(yCapture);
    if( ! xMatch.length() && ! yMatch.length() ){
      searchText += QStyleSheet::escape(line);
      searchText += "<br>";
      continue;
    }
    int xEnd = xPos + xMatch.length();
    int yEnd = yPos + yMatch.length();
//    cerr << "xPos \t" << xPos << "\t xEnd \t" << xEnd << " \t xMatch \t" << xMatch << "\t yPos \t" << yPos << "\t yEnd \t" << yEnd << "\t yMatch \t" << yMatch << endl;
    if( xPos <= yPos ){
      if(xPos>0) searchText += QStyleSheet::escape(line.left(xPos));
      if( xMatch.length() ){
        searchText += (colorBool ? "<font color=\"red\">" : "<font color=\"orange\">");
        searchText += QStyleSheet::escape(xMatch);
        searchText += "</font>";
      }
      if( yPos < xEnd ){
        searchText += QStyleSheet::escape(line.right(line.length() - xEnd));
        searchText += "<br>";
        continue;
      }
      searchText += QStyleSheet::escape(line.mid(xEnd, yPos-xEnd));
      if( yMatch.length() ){
        searchText += (colorBool ? "<font color=\"blueviolet\">" : "<font color=\"blue\">");
        searchText += QStyleSheet::escape(yMatch);
        searchText += "</font>";
      }
      searchText += QStyleSheet::escape(line.right(line.length() - yEnd));
    } else {
      if(yPos>0) searchText += QStyleSheet::escape(line.left(yPos));
      if( yMatch.length() ){

        searchText += (colorBool ? "<font color=\"blueviolet\">" : "<font color=\"blue\">");
        searchText += QStyleSheet::escape(yMatch);
        searchText += "</font>";
      }
      if( xPos < yEnd ){
        searchText += QStyleSheet::escape(line.right(line.length() - yEnd));
        searchText += "<br>";
        continue;
      }
      searchText += QStyleSheet::escape(line.mid(yEnd, xPos-yEnd));
      if( xMatch.length() ){
        searchText += (colorBool ? "<font color=\"red\">" : "<font color=\"orange\">");
        searchText += QStyleSheet::escape(xMatch);
        searchText += "</font>";
      }
      searchText += QStyleSheet::escape(line.right(line.length() - xEnd));
    }
    searchText += "<br>";
  }
  searchText += "</qt>";

//  cout << browser->text() << "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n";
//  cout << searchText << "\n\n\n\n\n\n";

  browser->setText(searchText);
}
void VarReader::readInput(){
  xRegExp->setPattern(  xLine->text() );
  yRegExp->setPattern(  yLine->text() );
  QString n = xCap->text().stripWhiteSpace();
  xCapture = (n.isEmpty() ? 0 : n.toInt());
  n = yCap->text().stripWhiteSpace();
  yCapture = (n.isEmpty() ? 0 : n.toInt());
//  cerr << "xCapture \t" << xCapture << "\t yCapture \t" << yCapture << endl;
}
void VarReader::apply(){
  readInput();
  switch(currentRadio){
    case regex:
      makeRegex();
      break;
    case column:
      if( ! tableFilled ) makeColumn();
      break;
    case edit:
//      makeEdit();
      break;
  }
}
void VarReader::readPoints(){
  if(currentRadio == regex){
    readInput();
    QTextStream stream(&text, IO_ReadOnly);
    QString line;
    while(1){
      line = stream.readLine();
      if( line.isNull() ) break;
      xRegExp->search(line);
      yRegExp->search(line);
      if(    xRegExp->pos(xCapture) < 0
          || yRegExp->pos(yCapture) < 0 ) continue;
      double x = xRegExp->cap(xCapture).toDouble();
      double y = yRegExp->cap(yCapture).toDouble();
      points->push_back(x, y);
    }
  } else {
    QRegExp splitter(  splitLine->text() );
    QTextStream stream(&text, IO_ReadOnly);
    QString line;
    QStringList list;
    int xNum = xColumn->text().toInt();
    int yNum = yColumn->text().toInt();
    if( xNum < 1 || yNum < 1 ) return;
    int biggerNum = ( xNum > yNum ? xNum : yNum );
    while(1){
      line = stream.readLine();
      if( line.isNull() ) break;
      list = QStringList::split(splitter, line);
      if( biggerNum > (int)list.size() ) continue;
      double x = list[xNum].toDouble();
      double y = list[yNum].toDouble();
//      cerr << " found " << x << " : " << y << endl;
      points->push_back(x, y);
    }
  }
}
