/***************************************************************************
                          xml_element.cpp  -  description
                             -------------------
    begin                : Mon Feb 3 2003
    copyright            : (C) 2003 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "xml_element.h"
#include "data.h"
#include "attribute_line_edit.h"
#include "grapheditview.h"
#include "domenumeration.h"
#include "collection.h"

#include <qregexp.h>
#include <qapplication.h>
#include <qfile.h>
#include <qmessagebox.h>

QString XML_Element::applicationDocType = "XML_Element";

QString XML_Element::spaceToBar(QString name){
  static QRegExp space("\\s");
  return name.replace(space, "_");
}

XML_Element::XML_Element(const QString elementName)
  : elementName(elementName),
    controlElementName(elementName),
    nameEditMode(EDIT),
    domElement(0),
    dummy(0)
{
//  cerr << " creating " << endl;
//  cerr << elementName << endl;
}
XML_Element::~XML_Element(){
//  cerr << tagName() << ":" << elementName << " is being destroyed.\n";
}
bool XML_Element::xml_read(QDomElement * element){
  xml_mode = ( element ? XML_READ : XML_INIT );
  domElement = element;
  xml_list();
  return true;
}
bool XML_Element::xml_write( QDomElement * parent ){
//  cerr << "adding " << tagName() << ":" << elementName << endl;
  xml_mode = XML_WRITE;
  QDomElement element = parent->ownerDocument().createElement(tagName());
  if( ! dummy){
    dummy = new QDomElement();
  }
  domElement = dummy;
  * domElement = element;
  xml_list();
  parent->appendChild(element);
  return true;
}
bool XML_Element::xml_shallow_write( QDomElement * parent ){
//  cerr << "adding " << tagName() << ":" << elementName << endl;
  xml_mode = XML_SHALLOW_WRITE;
  QDomElement element = parent->ownerDocument().createElement(tagName());
  if( ! dummy){
    dummy = new QDomElement();
  }
  domElement = dummy;
  * domElement = element;
  xml_list();
  parent->appendChild(element);
  return true;
}
bool XML_Element::xml_copy( XML_Element * other ){
  QDomDocument document(applicationDocType);
//  document.removeChild(document.documentElement());
  document.appendChild(document.createElement(applicationDocType));
  QDomElement root = document.documentElement();
  other->xml_write( & root );
  QDomElement myElement =
    root.elementsByTagName(tagName()).item(0).toElement();
  xml_read( & myElement );
  return true;
}
bool XML_Element::xml_edit(GraphEditView * editor){
  if( editor ) graphEditor = editor;
  if( ! graphEditor ) return false;
  graphEditor->clear();
  parentAttribute = new Attribute(xml_ID, graphEditor, tagName(), elementName);
  connect(this, SIGNAL(valueChanged()), parentAttribute, SLOT(valueChanged()));
  xml_mode = XML_EDIT;
  xml_list();
  parentAttribute->setOpen(true);
  return true;
}
bool XML_Element::xml_edit(Attribute * attribute, const bool subordinate ,
                          const EditMode edit){
  if(subordinate){
    parentAttribute = new Attribute(xml_ID, attribute, tagName(), elementName);
  } else {
    parentAttribute = attribute;
  }
  graphEditor = dynamic_cast< GraphEditView * >(attribute->listView());
  xml_mode = XML_EDIT;
  xml_list();
  if(edit == OPEN) parentAttribute->setOpen(true);
  return true;
}
void XML_Element::xml_list(){
  if( xml_mode != XML_INIT ){
    xml_qstring("name", elementName, controlElementName, nameEditMode); // name set in the constructor
  }
}

bool XML_Element::xml_load(QString fileName){
  QFile file(fileName);
  if( ! file.open(IO_ReadOnly)){
    xml_warning("can't read<br>" + fileName);
    return false;
  }
  QDomDocument document(applicationDocType);
//  document.removeChild(document.documentElement());
  document.appendChild(document.createElement(applicationDocType));
  QString error;
  int line, column;
  if( ! document.setContent( & file, &error, &line, &column ) ){
    QString message = "content error in " + fileName + "<br>";
    message += error + "<br> at ";
    message += line + ", " + column;
    xml_warning(message);
    file.close();
    return false;
  }
  file.close();
  QDomElement myElement =
    document.documentElement().elementsByTagName(tagName()).item(0).toElement();
  xml_read( & myElement );
  return true;
}
bool XML_Element::xml_save(QString fileName){
  QFile file(fileName);
  if( ! file.open(IO_WriteOnly)){
    xml_warning(fileName + "<br>is readonly");
    return false;
  }

  QDomDocument document(applicationDocType);
//  document.removeChild(document.documentElement());
  document.appendChild(document.createElement(applicationDocType));
  QDomElement root = document.documentElement();
  xml_write( & root );

  QTextStream stream( & file);
  stream << document.toString();
  file.close();
  return true;
}

bool XML_Element::makeXML_Element(XML_Element * element, QDomNodeList * list){
  for(unsigned int i=0; i<list->length(); i++){
    QDomElement myElement = list->item(i).toElement();
    if( myElement.attribute("name") == element->getElementName() ){
      element->xml_read( & myElement);
      return true;
    }
  }
  return false;
}
bool XML_Element::elementNameChanged(){
  if( elementName != controlElementName ){
    controlElementName = elementName;
    return true;
  }
  return false;
}
void XML_Element::xml_bool(const char * name, bool & val,
                           const bool & def, EditMode edit){
  switch(xml_mode){
    case XML_SHALLOW_WRITE:
    case XML_WRITE:
      domElement->setAttribute(spaceToBar(name), boolToQString(val));
      break;
    case XML_READ:
      val = domElement->attribute(spaceToBar(name), boolToQString(def)).lower() == "true";
      break;
    case XML_INIT:
      val = def;
      break;
    case XML_EDIT:
      if(edit != HIDE){
        new Attribute_bool(xml_ID, parentAttribute, name, val, def, edit);
      }
      break;
  }
//      cerr << name << " val: " << val << " def: " << def << endl;
}
void XML_Element::xml_int(const char * name, int & val,
                          const int & def, EditMode edit){
  switch(xml_mode){
    case XML_SHALLOW_WRITE:
    case XML_WRITE:
      domElement->setAttribute(spaceToBar(name), val);
      break;
    case XML_READ:
      val = domElement->attribute(spaceToBar(name), QString::number(def)).toInt();
      break;
    case XML_INIT:
      val = def;
      break;
    case XML_EDIT:
      if(edit != HIDE)
        new Attribute_int(xml_ID, parentAttribute, name, val, def, edit);
      break;
  }
//      cerr << name << " val: " << val << " def: " << def << endl;
}
void XML_Element::xml_double(const char * name, double & val,
                             const double & def, EditMode edit){
  switch(xml_mode){
    case XML_SHALLOW_WRITE:
    case XML_WRITE:
      domElement->setAttribute(spaceToBar(name), val);
      break;
    case XML_READ:
      val = domElement->attribute(spaceToBar(name), QString::number(def)).toDouble();
      break;
    case XML_INIT:
      val = def;
      break;
    case XML_EDIT:
      if(edit != HIDE)
        new Attribute_double(xml_ID, parentAttribute, name, val, def, edit);
      break;
  }
//      cerr << name << " val: " << val << " def: " << def << endl;
}
void XML_Element::xml_qstring(const char * name, QString & val,
                              const QString & def, EditMode edit){
  switch(xml_mode){
    case XML_SHALLOW_WRITE:
    case XML_WRITE:
      domElement->setAttribute(spaceToBar(name), val);
      break;
    case XML_READ:
      val = domElement->attribute(spaceToBar(name), def);
      break;
    case XML_INIT:
      val = def;
      break;
    case XML_EDIT:
      if(edit != HIDE)
        new Attribute_qstring(xml_ID, parentAttribute, name, val, def, edit);
      break;
  }
//      cerr << name << " val: " << val << " def: " << def << endl;
}
void XML_Element::xml_object(XML_Element * object, EditMode edit,
                              const bool subordinate){
  if( ! object ) return;
  switch(xml_mode){
    case XML_SHALLOW_WRITE:
      break;
    case XML_WRITE:
      if(object){
        object->xml_write( domElement );
      }
      break;
    case XML_READ:
      break;
    case XML_INIT:
      break;
    case XML_EDIT:
      if(edit != HIDE && object){
        object->xml_edit(parentAttribute, subordinate, edit);
      }
      break;
  }
}
void XML_Element::xml_enum(DomEnumeration * enumeration, const char * name,
                           int & val, const int def, EditMode edit,
                           XML_Element * element){
  switch(xml_mode){
    case XML_SHALLOW_WRITE:
    case XML_WRITE:
      domElement->setAttribute(spaceToBar(name),
                               enumeration->stringAtValue(val));
      break;
    case XML_READ:
      val = enumeration->valueAtString( domElement->attribute(spaceToBar(name),
                                            enumeration->stringAtValue(def)) );
      break;
    case XML_INIT:
      val = def;
      break;
    case XML_EDIT:
      if(edit != HIDE){
        Attribute * enumAttribute = new Attribute_enum(xml_ID, parentAttribute,
                                            enumeration, name, val, def, edit);
        if(element) element->xml_edit(enumAttribute, false,
                                   ( edit ) );
      }
      break;
  }
}
void XML_Element::xml_select(DomEnumeration * collection, const char * name,
                             int & val, const int & def, EditMode edit){
  switch(xml_mode){
    case XML_SHALLOW_WRITE:
    case XML_WRITE:
      domElement->setAttribute(spaceToBar(name), val);
      break;
    case XML_READ:
      val = domElement->attribute(spaceToBar(name), QString::number(def)).toInt();
      break;
    case XML_INIT:
      val = def;
      break;
    case XML_EDIT:
      if(edit != HIDE){
//        if ( edit == EDIT ) cerr << " edit " << getElementName();
//        else cerr << " don't edit " << getElementName();
//        if ( edit == SHOW ) cerr << " show " << getElementName();
//        else cerr << " don't show " << getElementName();
        Attribute * selectAttribute = new Attribute_select(xml_ID, parentAttribute,
                                             collection, name, val, def, edit);
        XML_Element * element = collection->elementAtValue(val);
        if(element) element->xml_edit(selectAttribute, false,
                                    ( edit ) );
      }
      break;
  }
}

bool XML_Element::xml_warning(QString message, bool cancel){
  if(cancel){
    return QMessageBox::Cancel ==
      QMessageBox::warning( qApp->mainWidget(),
                            QString::null, message,
                            QMessageBox::Ok,
                            QMessageBox::Cancel);
  } else {
    QMessageBox::warning( qApp->mainWidget(),
                          QString::null, message,
                          QMessageBox::Ok,
                          QMessageBox::NoButton);
    return false;
  }
}

