/***************************************************************************
                          xml_element.h  -  description
                             -------------------
    begin                : Mon Feb 3 2003
    copyright            : (C) 2003 by Volker Nannen
    email                : volker@nannen.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef XML_ELEMENT_H
#define XML_ELEMENT_H

#include <qobject.h>
#include <qstring.h>
#include <qdom.h>

#include <iostream>

using namespace std;

class GraphEditView;
class Attribute;
class DomEnumeration;
class QDomNodeList;
class QDomElement;

class XML_Element : public QObject {
  Q_OBJECT

public:
  enum EditMode {
    EDIT,
    SHOW,
    HIDE,
    OPEN
  };

  XML_Element(const QString elementName = QString::null);
  virtual ~XML_Element();

  virtual const QString tagName() const                { return "Undefined"; }
  QString getElementName()                             { return elementName; }
  void setElementName(QString newName)                 { elementName = newName; }

  bool xml_edit(GraphEditView * editor=0);
  bool xml_edit(Attribute * attribute, const bool subordinate = true,
              const EditMode edit = SHOW);

  virtual bool xml_write(QDomElement * parent);
  virtual bool xml_shallow_write(QDomElement * parent);
  virtual bool xml_read(QDomElement * element = 0);
  virtual void xml_list();
  virtual bool xml_save(QString fileName = QString::null);
  virtual bool xml_load(QString fileName = QString::null);
  virtual bool xml_copy(XML_Element * other);
  
protected:
  void xml_setID(const int newID)                      { xml_ID = newID; }
  
  bool makeXML_Element(XML_Element * element, QDomNodeList * list);
  
  static bool xml_warning(QString message, bool cancel = false);

  QDomElement * thisDomElement()                       { return domElement; }

  void xml_bool(const char *, bool &, const bool &, EditMode edit=SHOW);
  void xml_int(const char *, int &, const int &, EditMode edit=SHOW);
  void xml_double(const char *, double &, const double &, 
                  EditMode edit=SHOW);
  void xml_qstring(const char *, QString &, const QString &, 
                   EditMode edit=SHOW);
  void xml_object(XML_Element * object, EditMode edit=SHOW,
                  const bool subordinate=true);
  void xml_enum(DomEnumeration * enumeration, const char * name,
                int &, const int, EditMode edit=SHOW,
                XML_Element * element=0);
  void xml_select(DomEnumeration * enumeration, const char * name,
                  int & val, const int & def, EditMode edit=SHOW);

  bool elementNameChanged();
  void hideName()                                      { nameEditMode = HIDE; }

  static void setApplicationDocType(QString newDocType)
         { applicationDocType = newDocType; }
  static QString getApplicationDocType()
         { return applicationDocType; }

private:
  static QString spaceToBar(QString name);

public slots:
  virtual void refresh()                               {;}

signals:
  void valueChanged();

private:
  static QString applicationDocType;
  QString elementName, controlElementName;
  EditMode nameEditMode;
  QDomElement * domElement;
  QDomElement * dummy;
  enum XML_Mode {
    XML_READ,
    XML_WRITE,
    XML_SHALLOW_WRITE,
    XML_EDIT,
    XML_INIT
  } xml_mode;
  GraphEditView * graphEditor;
  Attribute * parentAttribute;

  int xml_ID;
};

#endif
